SystemOrganization addCategory: #'FileSystem-Core-Implementation'!
SystemOrganization addCategory: #'FileSystem-Core-Kernel'!
SystemOrganization addCategory: #'FileSystem-Core-Public'!

!String methodsFor: '*filesystem-core-converting' stamp: 'SeanDeNigris 7/12/2012 08:45'!
asFileReference

	^ FileSystem disk referenceTo: self! !

!String methodsFor: '*filesystem-core-converting' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
asPath
	"convert myself to a path"
	"Examples:
		'.' asPath
		'~/Desktop' asPath
		'/home/foo' asPath
		'../../foo.bar' asPath"
	^ FileSystem disk resolve: self! !

!String methodsFor: '*filesystem-core-converting' stamp: 'cwp 11/21/2009 11:30'!
asPathWith: anObject 
	^ anObject pathFromString: self! !

!String methodsFor: '*filesystem-core-converting' stamp: 'EstebanLorenzano 4/12/2012 14:30'!
asReference
	self deprecated: 'Use #asFileReference instead'  on: '12 April 2012' in:  'Pharo 1.4' .
	^ self asFileReference! !

!String methodsFor: '*filesystem-core-converting' stamp: 'StephaneDucasse 2/9/2011 13:41'!
asResolvedBy: aFileSystem
	^ aFileSystem resolveString: self! !

Notification subclass: #ResolutionRequest
	instanceVariableNames: 'origin'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!ResolutionRequest commentStamp: 'cwp 11/18/2009 11:38' prior: 0!
I represent a request for user assistance in resolving an origin. I am a resumable exception that gets raised when there is no way of automatically resolving a particular origin. !

!ResolutionRequest class methodsFor: 'instance creation' stamp: 'cwp 10/27/2009 10:13'!
for: origin
	^ self new
		origin: origin;
		signal! !

!ResolutionRequest methodsFor: 'exceptionDescription' stamp: 'CamilloBruni 7/10/2012 15:09'!
defaultAction
	| filedir ref |
	filedir := UIManager default chooseDirectory: 'Where is ', origin asString, '?'.
	ref := filedir ifNotNil: [FileSystem disk referenceTo: filedir fullName].
	self resume: ref! !

!ResolutionRequest methodsFor: 'accessing' stamp: 'cwp 10/27/2009 10:22'!
origin: aSymbol
	origin := aSymbol! !

!FileStream class methodsFor: '*FileSystem-Core' stamp: 'abc 5/11/2012 23:23'!
onHandle: aFileSystemHandle
	^ self concreteStream new
		open: aFileSystemHandle fullName
		forWrite: aFileSystemHandle isWritable! !

Error subclass: #FileSystemError
	instanceVariableNames: 'reference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!FileSystemError commentStamp: 'cwp 11/18/2009 12:32' prior: 0!
I am an abstract superclass for errors that may occur during filesystem operations.!

FileSystemError subclass: #DirectoryDoesNotExist
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!DirectoryDoesNotExist commentStamp: 'cwp 11/18/2009 12:33' prior: 0!
I am raised when I an operation is attempted inside a directory that does not exist. !

FileSystemError subclass: #DirectoryExists
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!DirectoryExists commentStamp: 'cwp 11/18/2009 12:35' prior: 0!
I am raised on an attempt to create a directory that already exists.!

FileSystemError subclass: #FileDoesNotExist
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!FileDoesNotExist commentStamp: 'cwp 11/18/2009 12:35' prior: 0!
I am raised when an operation is attempted on a file that does not exist. This includes cases where a file operation is attempted on a directory.!

FileSystemError subclass: #FileExists
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!FileExists commentStamp: 'cwp 11/18/2009 12:37' prior: 0!
I am raised on an attempt to create a file or directory over top of an existing file.!

!FileSystemError class methodsFor: 'instance creation' stamp: 'cwp 11/14/2009 23:32'!
reference: aReference
	^ self basicNew initializeWithReference: aReference! !

!FileSystemError class methodsFor: 'instance creation' stamp: 'cwp 11/14/2009 23:31'!
signalWith: aReference
	^ (self reference: aReference) signal! !

!FileSystemError methodsFor: 'initialize-release' stamp: 'lr 8/16/2010 16:00'!
initializeWithReference: aReference
	reference := aReference.
	messageText := aReference printString! !

!FileSystemError methodsFor: 'testing' stamp: 'lr 8/16/2010 16:00'!
isResumable
	^ true! !

!FileSystemError methodsFor: 'accessing' stamp: 'lr 7/13/2010 15:31'!
reference
	^ reference! !

FileSystemError subclass: #IllegalName
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!IllegalName class methodsFor: 'instance creation' stamp: 'DamienPollet 2/28/2011 17:04'!
name: aName
	^ self basicNew initializeWithName: aName! !

!IllegalName class methodsFor: 'instance creation' stamp: 'DamienPollet 2/28/2011 17:03'!
signalWith: aName
	^ (self name: aName) signal! !

!IllegalName methodsFor: 'initialization' stamp: 'DamienPollet 2/28/2011 17:08'!
initializeWithName: aName
	name := aName.
	self messageText: aName! !

!IllegalName methodsFor: 'accessing' stamp: 'DamienPollet 2/28/2011 17:08'!
name
	^ name! !

!Integer methodsFor: '*FileSystem-Core' stamp: 'CamilloBruni 7/10/2012 22:12'!
humanReadableSIByteSize
	^ String streamContents: [ :s|
		self humanReadableSIByteSizeOn: s ]! !

!Integer methodsFor: '*FileSystem-Core' stamp: 'CamilloBruni 7/10/2012 22:26'!
humanReadableSIByteSizeOn: s
	| exponent base |
	"Print a string with an SI binary unit represation of myself."
	base := 1000.
	self < base
		ifTrue: [ ^ s print: self; space; nextPut: $B ].
	exponent := (self log / base log) asInteger.
	(self / (base ** exponent)) printOn: s showingDecimalPlaces: 2.
	s 
		space;
		nextPut: ('kMGTPE' at: exponent);
		nextPut: $B.! !

Object subclass: #AbstractFileReference
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Public'!

!AbstractFileReference commentStamp: '<historical>' prior: 0!
I am an abstract superclass for FileLocator and FileReference. By implementing most of the methods on myself most code duplucation between the locator and the reference can be avoided!

!AbstractFileReference methodsFor: 'navigating' stamp: 'CamilloBruni 7/10/2012 15:08'!
, extension
	^ self withPath: self path, extension! !

!AbstractFileReference methodsFor: 'navigating' stamp: 'CamilloBruni 7/10/2012 20:36'!
/ aString
	^ self withPath: self path / aString! !

!AbstractFileReference methodsFor: 'comparing' stamp: 'CamilloBruni 7/10/2012 15:20'!
<= other
	^ self path <= self other path! !

!AbstractFileReference methodsFor: 'enumerating' stamp: 'CamilloBruni 7/10/2012 20:43'!
allChildren
	"Return all the files and folders recursively nested in the receiver"
	
	^ CollectVisitor breadthFirst: self resolve collect: [:entry | entry reference]! !

!AbstractFileReference methodsFor: 'enumerating' stamp: 'CamilloBruni 7/10/2012 20:46'!
allDirectories
	"Return all the directories recursively nested in the receiver."
	^ (SelectVisitor breadthFirst: self resolve select: [:entry | entry isDirectory ])
		collect: [ :each| each reference ]! !

!AbstractFileReference methodsFor: 'enumerating' stamp: 'CamilloBruni 7/10/2012 15:35'!
allEntries
	^ CollectVisitor breadthFirst: self resolve! !

!AbstractFileReference methodsFor: 'enumerating' stamp: 'CamilloBruni 7/10/2012 20:45'!
allFiles
	"Return all the files (not directories) recursively nested in the receiver."
	^ (SelectVisitor breadthFirst: self resolve select: [:entry | entry isFile ])
		collect: [ :each| each reference ]! !

!AbstractFileReference methodsFor: 'converting' stamp: 'CamilloBruni 7/10/2012 15:03'!
asAbsolute 
	self subclassResponsability! !

!AbstractFileReference methodsFor: 'converting' stamp: 'CamilloBruni 7/10/2012 15:04'!
asFileReference 
	self subclassResponsability ! !

!AbstractFileReference methodsFor: 'delegated' stamp: 'CamilloBruni 7/10/2012 15:04'!
asPathWith: anObject
	^ self resolve asPathWith: anObject! !

!AbstractFileReference methodsFor: 'resolving' stamp: 'CamilloBruni 7/10/2012 15:20'!
asResolvedBy: anObject
	^ anObject resolveReference: self! !

!AbstractFileReference methodsFor: 'converting' stamp: 'CamilloBruni 7/10/2012 15:03'!
asUrl
	"Convert my path into a file:// type url - a FileUrl."
	
	^ self path asUrl! !

!AbstractFileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 15:27'!
base
	"Returns the base of the basename, i.e. /foo/gloops.taz base is 'gloops'"
	^ self fullPath base! !

!AbstractFileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 15:28'!
basename
	"Returns the basename, i.e. /foo/gloops.taz basename is 'gloops.taz'"
	^ self fullPath basename! !

!AbstractFileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 15:07'!
basenameWithIndicator
	"Returns the basename with the indicator appended, i.e. /foo/gloops.taz basenameWithIndicator is 'gloops.taz', whereras /foo basenameWithIndicator is 'foo/'" 
	^ self basename, self indicator   ! !

!AbstractFileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 15:28'!
basenameWithoutExtension
	"Returns the basename, i.e. /foo/gloops.taz basenameWithoutExtension is 'gloops'"
	^ self fullPath basenameWithoutExtension! !

!AbstractFileReference methodsFor: 'private' stamp: 'CamilloBruni 7/10/2012 20:36'!
childGeneratorBlock: doBlock matching: patterns
	"
	FileDirectory default fileNamesMatching: '*'
	FileDirectory default fileNamesMatching: '*.image;*.changes'
	"
	
	| files reference|
	files := Set new.
	reference := self resolve.
	
	(patterns findTokens: ';', String crlf) do: [ :pattern | 
		doBlock value: reference value: [ :basename|
			(pattern match: basename)
				ifTrue: [ files add: (self / basename) ]]].
	
	^files asOrderedCollection! !

!AbstractFileReference methodsFor: 'enumerating' stamp: 'CamilloBruni 7/10/2012 20:37'!
children
	"Answer an array containing references to the direct children of this reference."
	| reference |
	reference := self resolve.
	^ (reference fileSystem childNamesAt: reference path) 
		collect: [ :aName | self / aName ]! !

!AbstractFileReference methodsFor: 'enumerating' stamp: 'CamilloBruni 7/10/2012 20:35'!
childrenMatching: patterns
	"
	FileSystem disk workingDirectory childrenMatching: '*'
	FileSystem disk workingDirectory childrenMatching: '*.image;*.changes'
	"
	^ self 
		childGeneratorBlock: [:reference : aBlock| 
			reference fileSystem childNamesAt: reference path do: aBlock ]
		matching:  patterns ! !

!AbstractFileReference methodsFor: 'comparing' stamp: 'CamilloBruni 7/10/2012 15:22'!
contains: anObject
	^ anObject isContainedBy: self resolve! !

!AbstractFileReference methodsFor: 'comparing' stamp: 'CamilloBruni 7/10/2012 15:32'!
containsPath: aPath
	^ self fullPath containsPath: aPath! !

!AbstractFileReference methodsFor: 'operations' stamp: 'CamilloBruni 7/10/2012 15:51'!
copyAllTo: aResolvable
	
	CopyVisitor copy: self resolve asAbsolute to: aResolvable resolve! !

!AbstractFileReference methodsFor: 'delegated' stamp: 'CamilloBruni 7/10/2012 15:04'!
copyTo: aReference
	^ self resolve copyTo: aReference resolve! !

!AbstractFileReference methodsFor: 'copying' stamp: 'CamilloBruni 7/10/2012 15:17'!
copyWithPath: newPath
	self subclassResponsability! !

!AbstractFileReference methodsFor: 'operations' stamp: 'CamilloBruni 7/10/2012 15:52'!
createDirectory
	self resolve createDirectory! !

!AbstractFileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 20:51'!
creationTime
	^ self resolve creationTime ! !

!AbstractFileReference methodsFor: 'operations' stamp: 'CamilloBruni 7/10/2012 15:04'!
delete
	^ self resolve delete! !

!AbstractFileReference methodsFor: 'operations' stamp: 'CamilloBruni 7/10/2012 15:52'!
deleteAll
	"delete this directory and all children of it"
	DeleteVisitor delete: self resolve! !

!AbstractFileReference methodsFor: 'operations' stamp: 'CamilloBruni 7/10/2012 15:52'!
deleteAllChildren
	"delete all children of the receiver"
	
	self children do: [:aReference | aReference deleteAll ]! !

!AbstractFileReference methodsFor: 'operations' stamp: 'CamilloBruni 7/10/2012 15:56'!
deleteIfAbsent: aBlock
	self resolve deleteIfAbsent: aBlock! !

!AbstractFileReference methodsFor: 'enumerating' stamp: 'CamilloBruni 7/10/2012 15:47'!
directories
	"Return all the directories (by opposition to files) contained in the receiver"

	| reference |
	reference := self resolve.
	^ (reference fileSystem directoryNamesAt: reference path) 
		collect: [ :aName | self withPath: self path / aName ]! !

!AbstractFileReference methodsFor: 'enumerating' stamp: 'CamilloBruni 7/10/2012 20:35'!
directoriesMatching: patterns
	"
	FileSystem disk workingDirectory directoriesMatching: '*'
	FileSystem disk workingDirectory directoriesMatching: 'package-cache'
	"
	^ self 
		childGeneratorBlock: [:reference : aBlock| 
			reference fileSystem directoryNamesAt: reference path do: aBlock ]
		matching:  patterns ! !

!AbstractFileReference methodsFor: 'operations' stamp: 'CamilloBruni 7/10/2012 15:54'!
ensureDeleted
	self deleteIfAbsent: [].! !

!AbstractFileReference methodsFor: 'operations' stamp: 'CamilloBruni 7/10/2012 15:04'!
ensureDirectory
	^ self resolve ensureDirectory! !

!AbstractFileReference methodsFor: 'operations' stamp: 'CamilloBruni 7/10/2012 15:53'!
ensureFile
	"Create if necessary a file for the receiver."
	self writeStream close.
	! !

!AbstractFileReference methodsFor: 'enumerating' stamp: 'CamilloBruni 7/10/2012 15:04'!
entries
	^ self resolve entries! !

!AbstractFileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 15:04'!
entry
	^ self resolve entry! !

!AbstractFileReference methodsFor: 'delegated' stamp: 'CamilloBruni 7/10/2012 15:04'!
exists
	^ self resolve exists! !

!AbstractFileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 15:30'!
extension
	
	^ self fullPath extension.! !

!AbstractFileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 15:27'!
extensions
	  "#('foo' 'foo.tar' 'foo.tar.gz' 'foo.1.tar' 'foo.1.tar.gz') collect: [:thing| thing extensions] => #(#() #('tar') #('tar' 'gz') #('1' 'tar') #('1' 'tar' 'gz'))"
	
        ^ self fullPath extensions! !

!AbstractFileReference methodsFor: 'enumerating' stamp: 'CamilloBruni 7/10/2012 15:30'!
fileNames
	^ self files collect: #basename! !

!AbstractFileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 15:01'!
fileSystem
	^ self resolve fileSystem! !

!AbstractFileReference methodsFor: 'enumerating' stamp: 'CamilloBruni 7/10/2012 15:47'!
files
	"Return all the files (as opposed to folders) contained in the receiver"
	
	| reference |
	reference := self resolve.
	^ (reference fileSystem fileNamesAt: reference path) 
		collect: [ :aName | self withPath: self path / aName ]! !

!AbstractFileReference methodsFor: 'enumerating' stamp: 'CamilloBruni 7/10/2012 20:35'!
filesMatching: patterns
	"
	FileSystem disk workingDirectory filesMatching: '*'
	FileSystem disk workingDirectory filesMatching: '*.image;*.changes'
	"
	^ self 
		childGeneratorBlock: [:reference : aBlock| 
			reference fileSystem fileNamesAt: reference path do: aBlock ]
		matching:  patterns ! !

!AbstractFileReference methodsFor: 'delegated' stamp: 'CamilloBruni 7/10/2012 15:04'!
filesystem
	self deprecated: 'Use fileSystem' on: '3 April 2012' in: 'Pharo 1.4'.
	^ self fileSystem! !

!AbstractFileReference methodsFor: 'delegated' stamp: 'CamilloBruni 7/10/2012 15:04'!
fullName

	^ self resolve fullName! !

!AbstractFileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 15:30'!
fullNameWithIndicator
	"Returns the basename with the indicator appended, i.e. /foo/gloops.taz basenameWithIndicator is '/foo/gloops.taz', whereras /foo basenameWithIndicator is '/foo/'" 
	^ self fullName, self indicator   ! !

!AbstractFileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 15:24'!
fullPath
	^ self subclassResponsability! !

!AbstractFileReference methodsFor: 'enumerating' stamp: 'CamilloBruni 7/10/2012 15:35'!
glob: aBlock
	^ SelectVisitor breadthFirst: self resolve select: aBlock! !

!AbstractFileReference methodsFor: 'testing' stamp: 'CamilloBruni 7/10/2012 15:05'!
hasChildren
	^self resolve hasChildren! !

!AbstractFileReference methodsFor: 'testing' stamp: 'CamilloBruni 7/10/2012 15:05'!
hasDirectories
	^self resolve hasDirectories! !

!AbstractFileReference methodsFor: 'testing' stamp: 'CamilloBruni 7/10/2012 15:05'!
hasFiles
	^self resolve hasFiles! !

!AbstractFileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 22:12'!
humanReadableSize
	^ self size humanReadableSIByteSize! !

!AbstractFileReference methodsFor: 'testing' stamp: 'CamilloBruni 7/10/2012 20:17'!
ifFile: fBlock ifDirectory: dBlock ifAbsent: aBlock
	^ self isFile
		ifTrue: fBlock
		ifFalse: [
			self isDirectory
				ifTrue: dBlock
				ifFalse: aBlock ]! !

!AbstractFileReference methodsFor: 'printing' stamp: 'CamilloBruni 7/10/2012 20:15'!
indicator
	"Returns a string indicating the type of reference:
	- '?' for a non existing reference',
	- '/' for a directory,
	- the empty string for a file."
	"When this framework gets more complete, it is possible to extend this behavior with the following indicators (taken from unix ls utility):
	- '*' for a regular file that is executable
	- '@' for a symbolic link
	- '|' for FIFOs
	- '=' for sockets
	- '>' for doors"
	^ self exists
		ifTrue: [self isDirectory ifTrue: ['/'] ifFalse: ['']  ]
		ifFalse: ['?']! !

!AbstractFileReference methodsFor: 'testing' stamp: 'CamilloBruni 7/10/2012 15:06'!
isAbsolute 
	self subclassResponsability ! !

!AbstractFileReference methodsFor: 'testing' stamp: 'CamilloBruni 7/10/2012 20:17'!
isChildOf: anObject
	^ self parent = anObject! !

!AbstractFileReference methodsFor: 'comparing' stamp: 'CamilloBruni 7/10/2012 15:33'!
isContainedBy: anObject
	^ anObject containsReference: self resolve! !

!AbstractFileReference methodsFor: 'testing' stamp: 'CamilloBruni 7/10/2012 15:04'!
isDirectory
	^ self resolve isDirectory! !

!AbstractFileReference methodsFor: 'testing' stamp: 'CamilloBruni 7/10/2012 15:04'!
isFile
	^ self resolve isFile! !

!AbstractFileReference methodsFor: 'testing' stamp: 'CamilloBruni 7/10/2012 15:04'!
isReadable
	^ self resolve isReadable! !

!AbstractFileReference methodsFor: 'testing' stamp: 'CamilloBruni 7/10/2012 15:06'!
isRelative 
	self subclassResponsability ! !

!AbstractFileReference methodsFor: 'testing' stamp: 'CamilloBruni 7/10/2012 15:04'!
isRoot
	^ self resolve isRoot! !

!AbstractFileReference methodsFor: 'testing' stamp: 'CamilloBruni 7/10/2012 20:19'!
isWritable
	^ self resolve isWritable! !

!AbstractFileReference methodsFor: 'delegated' stamp: 'CamilloBruni 7/10/2012 15:04'!
macTypeAndCreator
	^ self resolve macTypeAndCreator! !

!AbstractFileReference methodsFor: 'navigating' stamp: 'CamilloBruni 7/10/2012 15:49'!
makeRelative: anObject
	^ anObject relativeToReference: self resolve! !

!AbstractFileReference methodsFor: 'delegated' stamp: 'CamilloBruni 7/10/2012 15:04'!
mimeTypes
	^ self resolve mimeTypes! !

!AbstractFileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 20:51'!
modificationTime
	^ self resolve modificationTime ! !

!AbstractFileReference methodsFor: 'operations' stamp: 'CamilloBruni 7/10/2012 15:04'!
moveTo: aReference
	^ self resolve moveTo: aReference! !

!AbstractFileReference methodsFor: 'utility' stamp: 'CamilloBruni 7/10/2012 15:04'!
nextVersion
	^ self resolve nextVersion! !

!AbstractFileReference methodsFor: 'streams' stamp: 'CamilloBruni 7/10/2012 20:16'!
openWritable: aBoolean
	^ self resolve openWritable: aBoolean! !

!AbstractFileReference methodsFor: 'delegated' stamp: 'CamilloBruni 7/10/2012 15:04'!
parent
	^ self withPath: self path parent! !

!AbstractFileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 15:31'!
pathSegments
	^ self fullPath segments! !

!AbstractFileReference methodsFor: 'delegated' stamp: 'CamilloBruni 7/10/2012 15:04'!
pathString
	^ self resolve pathString! !

!AbstractFileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 21:29'!
permissions
	"Return the FileSystemPermission for this node"
	^ self resolve permissions! !

!AbstractFileReference methodsFor: 'streams' stamp: 'CamilloBruni 7/10/2012 15:26'!
readStream
	self subclassResponsability! !

!AbstractFileReference methodsFor: 'streams' stamp: 'CamilloBruni 7/10/2012 15:26'!
readStreamDo: aBlock
	| stream |
	stream := self readStream.
	^ [ aBlock value: stream ] 
		ensure: [ stream close ]! !

!AbstractFileReference methodsFor: 'streams' stamp: 'CamilloBruni 7/10/2012 15:26'!
readStreamDo: doBlock ifAbsent: absentBlock
	^ self isFile 
		ifTrue: [ self readStreamDo: doBlock ]
		ifFalse: absentBlock! !

!AbstractFileReference methodsFor: 'streams' stamp: 'CamilloBruni 7/10/2012 15:26'!
readStreamIfAbsent: absentBlock
	^ self isFile
		ifTrue: [ self readStream ]
		ifFalse: absentBlock! !

!AbstractFileReference methodsFor: 'navigating' stamp: 'CamilloBruni 7/10/2012 15:50'!
relativeTo: landmark
	"Answer a new path relative to landmark."
	
	"parent/child/grandChild relativeTo: parent returns child/grandChild
	(FSFilesystem disk / 'griffle' / 'plonk' / 'nurp') relativeTo: (FSFilesystem disk / 'griffle') 
	returns plonk/nurp"

	^ landmark makeRelative: self resolve! !

!AbstractFileReference methodsFor: 'navigating' stamp: 'CamilloBruni 7/10/2012 15:50'!
relativeToPath: landmarkPath 
	
	^ self fullPath relativeTo: landmarkPath! !

!AbstractFileReference methodsFor: 'navigating' stamp: 'CamilloBruni 7/10/2012 15:51'!
relativeToReference: landmarkReference
	"Return the path of the receiver relative to landmarkReference."
	
	^ self fullPath relativeTo: landmarkReference path! !

!AbstractFileReference methodsFor: 'operations' stamp: 'CamilloBruni 7/10/2012 17:35'!
renameTo: newBasename
	self subclassResponsability! !

!AbstractFileReference methodsFor: 'navigating' stamp: 'CamilloBruni 7/10/2012 15:05'!
resolve
	^ self subclassResponsability ! !

!AbstractFileReference methodsFor: 'navigating' stamp: 'CamilloBruni 7/10/2012 15:05'!
resolve: anObject
	^ anObject asResolvedBy: self! !

!AbstractFileReference methodsFor: 'navigating' stamp: 'CamilloBruni 7/10/2012 15:18'!
resolvePath: aPath
	^ self withPath: (self path resolvePath: aPath)! !

!AbstractFileReference methodsFor: 'navigating' stamp: 'CamilloBruni 7/10/2012 15:18'!
resolveReference: aReference
	^ aReference isAbsolute
		ifTrue: [ aReference ]
		ifFalse: [ self withPath: aReference path ]! !

!AbstractFileReference methodsFor: 'navigating' stamp: 'CamilloBruni 7/10/2012 15:19'!
resolveString: aString 
	self subclassResponsability! !

!AbstractFileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 22:05'!
size
	^ self resolve size! !

!AbstractFileReference methodsFor: 'streams' stamp: 'CamilloBruni 7/10/2012 22:34'!
streamWritable: writable do: aBlock
	^ writable 
		ifTrue: [ self writeStreamDo: aBlock ]
		ifFalse: [ self readStreamDo: aBlock ]! !

!AbstractFileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 15:31'!
uri
	"Convert my path into a file:// type url. For odd characters use %20 notation."

	self flag: 'TODO: return a real URI object instead of a string'.
	^(String streamContents: [:strm |
		strm nextPutAll: 'file:'.
		self pathSegments do: [:each | strm nextPut: $/; nextPutAll: each encodeForHTTP].
		strm nextPut: $/]) asURI! !

!AbstractFileReference methodsFor: 'navigating' stamp: 'CamilloBruni 7/10/2012 15:05'!
withExtension: aString
	^ self withPath: (self path withExtension: aString)! !

!AbstractFileReference methodsFor: 'navigating' stamp: 'CamilloBruni 7/10/2012 15:18'!
withPath: newPath
	^ self path == newPath
		ifTrue: [ self ]
		ifFalse: [ self copyWithPath: newPath ]! !

!AbstractFileReference methodsFor: 'streams' stamp: 'CamilloBruni 7/10/2012 15:25'!
writeStream
	self subclassResponsability! !

!AbstractFileReference methodsFor: 'streams' stamp: 'CamilloBruni 7/10/2012 15:24'!
writeStreamDo: aBlock
	| stream |
	stream := self writeStream.
	^ [ aBlock value: stream ]
		ensure: [ stream close ]! !

!AbstractFileReference methodsFor: 'streams' stamp: 'CamilloBruni 7/10/2012 15:24'!
writeStreamDo: doBlock ifPresent: presentBlock
	^ self isFile
		ifTrue: presentBlock
		ifFalse: [ self writeStreamDo: doBlock ]! !

!AbstractFileReference methodsFor: 'streams' stamp: 'CamilloBruni 7/10/2012 15:24'!
writeStreamIfPresent: presentBlock
	^ self isFile 
		ifTrue: presentBlock
		ifFalse: [ self writeStream ]! !

AbstractFileReference subclass: #FileLocator
	instanceVariableNames: 'origin path'
	classVariableNames: 'Resolver'
	poolDictionaries: ''
	category: 'FileSystem-Core-Public'!

!FileLocator commentStamp: '<historical>' prior: 0!
I am a late-bound reference. I refer to a file or directory in relation to a well-known location on the filesystem, called an origin. When asked to perform concrete operation, I look up the current location of my origin, and resolve my path against it. 

Usage
----------
FileLocator vmDirectory parent pathString
	> '/Applications'


FileLocator desktop.
FileLocator desktop basename.

FileLocator home basename.
FileLocator image.
FileLocator vmBinary asAbsolute pathString 
	>  '/Applications/CogVM.app/Contents/MacOS/CogVM'
FileLocator vmBinary pathString 
	> '/Applications/CogVM.app/Contents/MacOS/CogVM'
		




Implementation
------------------------
origin 
	A symbolic name for base reference I use to resolve myself.

path
	A relative path that is resolved against my origin"
	
	!

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
A
	^ self driveNamed: #A 
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
B
	^ self driveNamed: #B
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
C
	^ self driveNamed: #C
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
D
	^ self driveNamed: #D
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
E
	^ self driveNamed: #E
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
F
	^ self driveNamed: #F
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
G
	^ self driveNamed: #G
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
H
	^ self driveNamed: #H
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
I
	^ self driveNamed: #I
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
J
	^ self driveNamed: #J
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
K
	^ self driveNamed: #K
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
L
	^ self driveNamed: #L
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
M
	^ self driveNamed: #M
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
N
	^ self driveNamed: #N
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
O
	^ self driveNamed: #O
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
P
	^ self driveNamed: #P
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
Q
	^ self driveNamed: #Q
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
R
	^ self driveNamed: #R
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
S
	^ self driveNamed: #S
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
T
	^ self driveNamed: #T
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
U
	^ self driveNamed: #U
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
V
	^ self driveNamed: #V
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
W
	^ self driveNamed: #W! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
X
	^ self driveNamed: #X
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
Y
	^ self driveNamed: #Y
! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:38'!
Z
	^ self driveNamed: #Z
! !

!FileLocator class methodsFor: 'class initialization' stamp: 'cwp 10/26/2009 20:54'!
addResolver: aResolver
	Resolver addResolver: aResolver! !

!FileLocator class methodsFor: 'origins' stamp: 'lr 7/13/2010 13:29'!
changes
	^ self origin: #changes ! !

!FileLocator class methodsFor: 'origins' stamp: 'CamilloBruni 7/10/2012 21:39'!
cwd
	^ self workingDirectory! !

!FileLocator class methodsFor: 'origins' stamp: 'cwp 10/27/2009 10:24'!
desktop
	^ self origin: #desktop! !

!FileLocator class methodsFor: 'origins' stamp: 'CamilloBruni 5/24/2012 12:07'!
documents
	^ self origin: #documents! !

!FileLocator class methodsFor: 'windows-origins' stamp: 'CamilloBruni 7/10/2012 21:39'!
driveNamed: driveName
	^ FileReference fileSystem: (FileSystem disk) path: Path / (driveName, ':')! !

!FileLocator class methodsFor: 'class initialization' stamp: 'cwp 10/27/2009 10:28'!
flushCaches
	Resolver flushCaches! !

!FileLocator class methodsFor: 'origins' stamp: 'cwp 10/27/2009 09:34'!
home
	^ self origin: #home! !

!FileLocator class methodsFor: 'origins' stamp: 'cwp 10/25/2009 09:54'!
image
	^ self origin: #image ! !

!FileLocator class methodsFor: 'origins' stamp: 'lr 7/13/2010 13:35'!
imageDirectory
	^ self origin: #imageDirectory ! !

!FileLocator class methodsFor: 'class initialization' stamp: 'cwp 11/20/2009 15:01'!
initialize
	Smalltalk addToStartUpList: self.
	self startUp: true! !

!FileLocator class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
origin: aSymbol 
	^ self origin: aSymbol path: Path workingDirectory! !

!FileLocator class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/24/2012 14:49'!
origin: aSymbol path: aPath
	^ self basicNew 
			initializeWithOrigin: aSymbol path: aPath! !

!FileLocator class methodsFor: 'origins' stamp: 'CamilloBruni 5/24/2012 12:07'!
preferences
	^ self origin: #preferences! !

!FileLocator class methodsFor: 'origins' stamp: 'CamilloBruni 7/10/2012 21:36'!
root
	^ FileSystem disk root! !

!FileLocator class methodsFor: 'class initialization' stamp: 'EstebanLorenzano 4/2/2012 11:39'!
startUp: resuming
	resuming
		ifFalse: [ ^ self ].
	Resolver := InteractiveResolver new.
	Resolver addResolver: SystemResolver new.
	Resolver addResolver: PlatformResolver forCurrentPlatform! !

!FileLocator class methodsFor: 'accessing' stamp: 'cwp 10/27/2009 11:25'!
supportedOrigins
	| origins current |
	origins := IdentitySet new.
	current := Resolver.
	[current notNil] whileTrue:
		[origins addAll: current supportedOrigins.
		current := current next].
	^ origins! !

!FileLocator class methodsFor: 'origins' stamp: 'cwp 10/26/2009 11:37'!
vmBinary
	^ self origin: #vmBinary! !

!FileLocator class methodsFor: 'origins' stamp: 'cwp 10/26/2009 13:49'!
vmDirectory
	^ self origin: #vmDirectory! !

!FileLocator class methodsFor: 'origins' stamp: 'CamilloBruni 7/10/2012 21:36'!
workingDirectory
	^ FileSystem disk referenceTo: RelativePath new! !

!FileLocator methodsFor: 'comparing' stamp: 'cwp 10/26/2009 10:28'!
= other
	^ self species = other species
		and: [origin = other origin
			and: [path = other path]]! !

!FileLocator methodsFor: 'converting' stamp: 'cwp 10/25/2009 10:30'!
asAbsolute
	^ self ! !

!FileLocator methodsFor: 'converting' stamp: 'EstebanLorenzano 4/12/2012 14:28'!
asFileReference
	^ self resolve! !

!FileLocator methodsFor: 'copying' stamp: 'CamilloBruni 7/10/2012 15:17'!
copyWithPath: newPath
	^ self class origin: origin path: newPath! !

!FileLocator methodsFor: 'error handling' stamp: 'CamilloBruni 5/24/2012 12:35'!
doesNotUnderstand: aMessage
	"Redirect message to the resolved version of this FileLocator.
	If FileReference won't understand the message send a normal DNU."

	| resolved |
	
	resolved := self resolve.
	(resolved respondsTo: aMessage selector)
		ifTrue: [ ^ resolved perform: aMessage selector withArguments: aMessage arguments ].
	
	^ super doesNotUnderstand: aMessage.! !

!FileLocator methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 15:23'!
fullPath
	^ self resolve path! !

!FileLocator methodsFor: 'comparing' stamp: 'cwp 10/25/2009 11:05'!
hash
	^ origin hash bitXor: path hash! !

!FileLocator methodsFor: 'initialize-release' stamp: 'cwp 10/25/2009 09:56'!
initializeWithOrigin: aSymbol path: aPath
	self initialize.
	origin := aSymbol.
	path := aPath.! !

!FileLocator methodsFor: 'testing' stamp: 'cwp 10/25/2009 10:30'!
isAbsolute
	^ true! !

!FileLocator methodsFor: 'testing' stamp: 'cwp 10/25/2009 11:15'!
isRelative
	^ false! !

!FileLocator methodsFor: 'accessing' stamp: 'cwp 10/25/2009 21:31'!
origin
	^ origin! !

!FileLocator methodsFor: 'accessing' stamp: 'cwp 10/25/2009 21:31'!
path
	^ path! !

!FileLocator methodsFor: 'printing' stamp: 'EstebanLorenzano 4/3/2012 12:55'!
printOn: aStream
	| fs |
	aStream
		nextPut: ${;
		nextPutAll: origin;
		nextPut: $}.
	path isWorkingDirectory
		ifTrue: [ ^ self ].
	fs := self fileSystem.
	aStream nextPut: fs delimiter.
	fs printPath: path on: aStream! !

!FileLocator methodsFor: 'streams' stamp: 'CamilloBruni 7/10/2012 15:26'!
readStream
	^ self resolve readStream ! !

!FileLocator methodsFor: 'operations' stamp: 'CamilloBruni 7/10/2012 17:45'!
renameTo: newBasename
	
	| result |
	result := self resolve renameTo: newBasename.
	path size > 0
		ifTrue: [ path basename: newBasename ]
		ifFalse: [ path := result path ]! !

!FileLocator methodsFor: 'navigating' stamp: 'cwp 10/25/2009 09:59'!
resolve
	^ (Resolver resolve: origin) resolve: path! !

!FileLocator methodsFor: 'navigating' stamp: 'CamilloBruni 7/10/2012 15:19'!
resolveString: aString
	| filesystem thePath |
	filesystem := (Resolver resolve: origin) fileSystem.
	thePath := filesystem pathFromString: aString.
	^ self withPath: (path resolvePath: thePath)! !

!FileLocator methodsFor: 'streams' stamp: 'CamilloBruni 7/10/2012 15:26'!
writeStream
	^ self resolve writeStream ! !

AbstractFileReference subclass: #FileReference
	instanceVariableNames: 'filesystem path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Public'!

!FileReference commentStamp: '<historical>' prior: 0!
I combine a filesystem and path, which is sufficient to refer to a concrete file or directory. I provide methods for navigating my filesystem, performing filesystem operations and opening and closing files.  I am the primary mechanism for working with files and directories. 

| working |
working := FileSystem disk workingDirectory.
working files 

| disk |
disk := FileSystem disk.
disk root.                               	"a reference to the root directory"
disk working.                         	"a reference to the working directory"!

!FileReference class methodsFor: 'cross platform' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
/ aString
	"Answer a reference to the argument resolved against the root of the current disk filesystem."
	
	^ FileSystem disk / aString! !

!FileReference class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 4/3/2012 13:21'!
fileSystem: aFilesystem path: aPath
	^ self new setFileSystem: aFilesystem path: aPath! !

!FileReference methodsFor: 'comparing' stamp: 'EstebanLorenzano 4/3/2012 13:02'!
= other
	^ self species = other species
		and: [self path = other path
			and: [self fileSystem = other fileSystem]]! !

!FileReference methodsFor: 'converting' stamp: 'CamilloBruni 7/10/2012 15:03'!
asAbsolute
	"Return the receiver as an absolute file reference."
	
	^ self isAbsolute
		ifTrue: [ self ]
		ifFalse: [ filesystem referenceTo: (filesystem resolve: path) ]! !

!FileReference methodsFor: 'converting' stamp: 'EstebanLorenzano 4/12/2012 14:28'!
asFileReference
	^ self! !

!FileReference methodsFor: 'comparing' stamp: 'cwp 10/26/2009 01:03'!
contains: anObject
	^ anObject isContainedBy: self! !

!FileReference methodsFor: 'comparing' stamp: 'EstebanLorenzano 4/3/2012 13:05'!
containsReference: aReference
	^  aReference fileSystem = filesystem and: [path contains: aReference path]! !

!FileReference methodsFor: 'operations' stamp: 'EstebanLorenzano 4/3/2012 13:05'!
copyTo: aReference
	self isDirectory
		ifTrue: [ aReference ensureDirectory ]
		ifFalse: [ 
			filesystem = aReference fileSystem
				ifTrue: [ filesystem copy: path to: aReference path ]
				ifFalse: [ filesystem copy: path toReference: aReference ] ]! !

!FileReference methodsFor: 'copying' stamp: 'CamilloBruni 7/10/2012 15:17'!
copyWithPath: newPath
	^ filesystem referenceTo: newPath! !

!FileReference methodsFor: 'operations' stamp: 'cwp 11/17/2009 21:05'!
createDirectory
	filesystem createDirectory: path! !

!FileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 20:50'!
creationTime 
	^ filesystem creationTime: self path! !

!FileReference methodsFor: 'operations' stamp: 'cwp 7/22/2009 07:42'!
delete
	filesystem delete: path! !

!FileReference methodsFor: 'operations' stamp: 'CamilloBruni 7/10/2012 15:52'!
deleteIfAbsent: aBlock
	self exists 
		ifTrue: [ self delete ]
		ifFalse: aBlock! !

!FileReference methodsFor: 'operations' stamp: 'sd 2/11/2011 20:16'!
ensureDirectory
	"Create if necessary a directory for the receiver."
	filesystem ensureDirectory: path
	! !

!FileReference methodsFor: 'navigating' stamp: 'EstebanLorenzano 4/3/2012 13:07'!
entries
	"Return the entries (meta data - file description) of the direct children of the receiver"
	^ self fileSystem entriesAt: self path ! !

!FileReference methodsFor: 'accessing' stamp: 'sd 2/11/2011 19:58'!
entry
	"Return the entry (meta data) describing the receiver."
	^ filesystem entryAt: path! !

!FileReference methodsFor: 'testing' stamp: 'cwp 1/13/2009 20:52'!
exists
	^ filesystem exists: path! !

!FileReference methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/3/2012 13:03'!
fileSystem
	"Return the filesystem to which the receiver belong."
	^ filesystem! !

!FileReference methodsFor: 'accessing' stamp: 'sd 2/11/2011 20:34'!
fullName
	"Return the full path name of the receiver."
	^ filesystem stringFromPath: (filesystem resolve: path)! !

!FileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 15:23'!
fullPath
	^ self path! !

!FileReference methodsFor: 'testing' stamp: 'StephaneDucasse 6/22/2012 17:32'!
hasChildren
	"Return whether the receiver has any children."
	"FileSystem workingDirectory hasChildren"
	
	^ filesystem hasChildren: path! !

!FileReference methodsFor: 'testing' stamp: 'CamilloBruni 7/10/2012 20:17'!
hasDirectories
	"Return whether the receiver has children that are directories."
	"FileSystem workingDirectory hasDirectories"
	
	^ filesystem hasDirectories: path! !

!FileReference methodsFor: 'testing' stamp: 'StephaneDucasse 6/22/2012 17:35'!
hasFiles
	"Return whether the receiver has children that are files."
	"FileSystem workingDirectory hasFiles"
	
	^ filesystem hasFiles: path! !

!FileReference methodsFor: 'comparing' stamp: 'cwp 9/16/2009 23:54'!
hash
	^ path hash bitXor: filesystem hash! !

!FileReference methodsFor: 'testing' stamp: 'cwp 7/20/2009 09:24'!
isAbsolute
	^ path isAbsolute! !

!FileReference methodsFor: 'testing' stamp: 'cwp 1/13/2009 21:39'!
isDirectory
	^ filesystem isDirectory: path! !

!FileReference methodsFor: 'testing' stamp: 'cwp 1/13/2009 21:57'!
isFile
	^ filesystem isFile: path! !

!FileReference methodsFor: 'testing' stamp: 'CamilloBruni 5/9/2012 00:37'!
isReadable
	^ filesystem isReadable: path! !

!FileReference methodsFor: 'testing' stamp: 'cwp 7/20/2009 09:25'!
isRelative
	^ path isRelative! !

!FileReference methodsFor: 'testing' stamp: 'cwp 7/20/2009 09:26'!
isRoot
	^ path isRoot! !

!FileReference methodsFor: 'testing' stamp: 'CamilloBruni 5/9/2012 00:37'!
isWritable
	^ filesystem isWritable: path! !

!FileReference methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 11:37'!
macTypeAndCreator
	| results typeString creatorString |
	"get the Macintosh file type and creator info for the file with the given name. Fails if the file does not exist or if the type and creator type arguments are not strings of length 4. Does nothing on other platforms (where the underlying primitive is a noop)."
	"FileDirectory default getMacFileNamed: 'foo'"

	self flag: 'TODO'.
	"
	typeString := ByteArray new: 4 withAll: ($? asInteger). #[63 63 63 63]
	creatorString := ByteArray new: 4 withAll: ($? asInteger).
	[self primGetMacFileNamed: (self fullNameFor: fileName) asVmPathName
		type: typeString
		creator: creatorString.] ensure: 
		[typeString := typeString asString. 
		creatorString := creatorString asString].
	results := Array with: typeString convertFromSystemString with: creatorString convertFromSystemString.
	^results"
! !

!FileReference methodsFor: 'accessing' stamp: 'CamilloBruni 5/9/2012 13:04'!
mimeTypes
	"Return the possible mime types for the given path."
	^ filesystem mimeTypesAt: path! !

!FileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 20:51'!
modificationTime 
	^ filesystem modificationTime: self path! !

!FileReference methodsFor: 'operations' stamp: 'EstebanLorenzano 4/3/2012 13:08'!
moveTo: aReference
	
	| result |
	result := self fileSystem rename: self path to: aReference path.
	result ifNotNil: [
		self setFileSystem: filesystem path: aReference path ].		! !

!FileReference methodsFor: 'utility' stamp: 'GuillermoPolito 6/22/2012 13:31'!
nextVersion
	"Assumes a file name includes a version number encoded as '.' followed by digits 
	preceding the file extension.  Increment the version number and answer the new file name.
	If a version number is not found, return just the file"

	| parent version versionNumbers nameWithoutExtension |
	
	self exists
		ifFalse: [ ^ self ].
	
	parent := self parent.
	nameWithoutExtension := self basename copyUpTo: $..
	
	versionNumbers := parent files 
				select: [ :f| 
					(f basename beginsWith: nameWithoutExtension) ]
				thenCollect: [ :f| 
					Number squeezeNumberOutOfString: f basename ifFail: [ 0 ]].
	
	versionNumbers ifEmpty: [ ^self ].
	
	version := versionNumbers max + 1.
	^ parent / (nameWithoutExtension , '.', version asString) , self extension! !

!FileReference methodsFor: 'streams' stamp: 'CamilloBruni 1/20/2012 13:24'!
openWritable: aBoolean 
	^ filesystem open: path writable: aBoolean! !

!FileReference methodsFor: 'accessing' stamp: 'CamilloBruni 1/19/2012 14:16'!
path
	"Return the path internal representation that denotes the receiver in the context of its filesystem. "
	^ path! !

!FileReference methodsFor: 'deprecated' stamp: 'StephaneDucasse 6/22/2012 19:00'!
pathName
	
	self deprecated: 'Use fullName instead' on: 'StephaneDucasse 6/22/2012 19:00'  in: #Pharo20.
	^ self fullName. ! !

!FileReference methodsFor: 'printing' stamp: 'sd 2/11/2011 20:34'!
pathString
	"Return the full path name of the receiver."
	
	^ filesystem stringFromPath: (filesystem resolve: path)! !

!FileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 21:29'!
permissions
	^ filesystem permissions: self path! !

!FileReference methodsFor: 'printing' stamp: 'cwp 10/11/2009 22:32'!
printOn: aStream
	filesystem forReferencePrintOn: aStream.
	filesystem printPath: path on: aStream! !

!FileReference methodsFor: 'streams' stamp: 'CamilloBruni 7/10/2012 15:25'!
readStream
	^ filesystem readStreamOn: self path! !

!FileReference methodsFor: 'operations' stamp: 'SeanDeNigris 5/12/2012 11:14'!
renameTo: newBasename
	
	| destinationPath |
	destinationPath := self fileSystem rename: self to: self parent / newBasename.
	
	destinationPath ifNotNil: [
		self 
			setFileSystem: filesystem 
			path: destinationPath ].
	^ self
		! !

!FileReference methodsFor: 'accessing' stamp: 'cwp 10/26/2009 02:02'!
resolve
	^ self! !

!FileReference methodsFor: 'resolving' stamp: 'cwp 10/26/2009 01:03'!
resolve: anObject
	^ anObject asResolvedBy: self! !

!FileReference methodsFor: 'resolving' stamp: 'CamilloBruni 1/19/2012 12:45'!
resolvePath: anObject
	^ self withPath: (path resolve: anObject)! !

!FileReference methodsFor: 'resolving' stamp: 'EstebanLorenzano 4/3/2012 13:09'!
resolveReference: aReference
	
	^ (filesystem = aReference fileSystem or: [aReference isRelative])
		ifTrue: [filesystem referenceTo: (path resolvePath: aReference path)]
		ifFalse: [aReference]! !

!FileReference methodsFor: 'resolving' stamp: 'cwp 11/21/2009 11:30'!
resolveString: aString 
	| thePath |
	thePath := filesystem pathFromString: aString.
	^ filesystem referenceTo: (path resolve: thePath)! !

!FileReference methodsFor: 'initialize-release' stamp: 'EstebanLorenzano 4/3/2012 13:07'!
setFileSystem: aFilesystem path: aPath
	filesystem := aFilesystem.
	path := aPath! !

!FileReference methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 02:01'!
setMacType: aTypeString creator: aCreatorString
	self flag: 'TODO'.
	
" 	self primSetMacFileNamed: (self fullNameFor: fileName) asVmPathName
		type: typeString convertToSystemString
		creator: creatorString convertToSystemString.
"! !

!FileReference methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 22:06'!
size
	^ filesystem size: path! !

!FileReference methodsFor: 'streams' stamp: 'CamilloBruni 7/10/2012 15:25'!
writeStream
	^ filesystem writeStreamOn: self path! !

Object subclass: #FileSystem
	instanceVariableNames: 'workingDirectory store'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Public'!

!FileSystem commentStamp: '<historical>' prior: 0!
I present a low-level protocol for interacting with filesystems. I hold a reference to
a store (a subinstance of FileSystemStore) which takes care of the details of performing 
file and directory operations on the filesystem I represent. 

I keep track of the current directory, and am responsible for resolving all paths that
I pass into my store. My store acts as a factory and offers platform specific actions.


FileSystem instances know two methods that return a Reference object: workingDirectory and root.

FileSystem disk workingDirectory
FileSystem disk root

!

!FileSystem class methodsFor: 'initializing' stamp: 'EstebanLorenzano 4/3/2012 13:13'!
startUp: aBoolean
	"This is only here to deal with migration from older versions of
	FileSystem that wanted to receive startup notifcations."
	
	Smalltalk removeFromStartUpList: self! !

!FileSystem class methodsFor: 'instance creation' stamp: 'cwp 2/18/2011 20:34'!
store: aStore
	^ self basicNew
		initializeWithStore: aStore;
		yourself! !

!FileSystem methodsFor: 'navigating' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
* anObject
	"Return a relative reference."
	
	^ self referenceTo:( Path * anObject)! !

!FileSystem methodsFor: 'navigating' stamp: 'cwp 3/25/2011 13:04'!
/ anObject
	"Return the absolute reference obtained by resolving anObject against the
	root of this filesystem."
	
	^ self root / anObject! !

!FileSystem methodsFor: 'comparing' stamp: 'cwp 2/18/2011 16:08'!
= other
	^ self species = other species and: [self store = other store]! !

!FileSystem methodsFor: 'accessing' stamp: 'cwp 3/29/2011 15:58'!
changeDirectory: aPath
	self workingDirectoryPath: (self resolve: aPath)! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 5/10/2012 16:01'!
checkName: aString fixErrors: fixErrors
	^ store checkName: aString fixErrors: fixErrors! !

!FileSystem methodsFor: 'public-enumerating' stamp: 'CamilloBruni 7/10/2012 15:43'!
childNamesAt: aResolvable
	^ Array streamContents: [ :out | 
		self childNamesAt: aResolvable do: [ :path|
			out nextPut: path ]].! !

!FileSystem methodsFor: 'public-enumerating' stamp: 'CamilloBruni 7/10/2012 15:43'!
childNamesAt: aResolvable do: aBlock
	| path |
	path := self resolve: aResolvable.
	store
		directoryAt: path
		ifAbsent: [ store signalDirectoryDoesNotExist: path ]
		nodesDo: [ :entry | 
			aBlock value: (store basenameFromEntry: entry) ]! !

!FileSystem methodsFor: 'public-enumerating' stamp: 'CamilloBruni 1/19/2012 00:31'!
childrenAt: aResolvable
	^ Array streamContents: [ :out | 
		self childrenAt: aResolvable do: [ :path|
			out nextPut: path ]].! !

!FileSystem methodsFor: 'public-enumerating' stamp: 'CamilloBruni 6/22/2012 21:58'!
childrenAt: aResolvable do: aBlock
	| path |
	path := self resolve: aResolvable.
	store
		directoryAt: path
		ifAbsent: [ store signalDirectoryDoesNotExist: path ]
		nodesDo: [ :entry | 
			aBlock value: path / (store basenameFromEntry: entry) ]! !

!FileSystem methodsFor: 'public' stamp: 'cwp 2/19/2011 01:39'!
close
	store close! !

!FileSystem methodsFor: 'public' stamp: 'EstebanLorenzano 4/3/2012 13:29'!
copy: sourcePath ifAbsent: absentBlock to: destinationPath ifPresent: presentBlock
	"Copy the file referenced as sourcePath to the destination referred as destPath. Perform associate actions in case of problems."

	store
		copy: (self resolve: sourcePath)
		ifAbsent: absentBlock
		to: (self resolve: destinationPath)
		ifPresent: presentBlock
		fileSystem: self! !

!FileSystem methodsFor: 'public' stamp: 'cwp 4/3/2011 22:17'!
copy: sourcePath to: destPath
	"Copy the file referenced as sourcePath to the destination referred as destPath.  
	If there is no file at sourcePath, raise FileDoesNotExist.
	If destPath is a file, raise FileExists."
	
	self
		copy: sourcePath
		ifAbsent: [ store signalFileDoesNotExist: sourcePath ]
		to: destPath
		ifPresent: [ store signalFileExists: destPath ]! !

!FileSystem methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:06'!
copy: aPath toReference: destRef
	| inputStream path |
	
	path := self resolve: aPath.
	
	[
	inputStream := self readStreamOn: path.
	inputStream ifNil: [ store signalFileDoesNotExist: path ].
	destRef fileSystem copyFrom: inputStream to: destRef path 
	
		] ensure: [ inputStream ifNotNil: [ inputStream close ]]! !

!FileSystem methodsFor: 'private' stamp: 'CamilloBruni 1/20/2012 12:33'!
copyFrom: inputStream to: destPath
	| buffer out |
	out := nil.
	(self exists: destPath)
		ifTrue: [ store signalFileExists: destPath ].
	^ [ 
	out := self writeStreamOn: destPath.
	buffer := ByteArray new: 1024.
	[ inputStream atEnd ]
		whileFalse: [ 
			buffer := inputStream nextInto: buffer.
			out nextPutAll: buffer ]] ensure: [ out ifNotNil: [ out close ]]! !

!FileSystem methodsFor: 'public' stamp: 'cwp 3/25/2011 13:15'!
createDirectory: aResolvable
	"Resolve aResolvable into an absolute path, then as the store to create a directory there. 
	The store is expected to raise an exception if it cannot do so."

	^ store createDirectory: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 7/10/2012 20:54'!
creationTime: aResolvable
	"Resolve the argument, and answer true if the result refers
	to a directory, false if it refers to a file or doesn't exist."

	^ store creationTime: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'public' stamp: 'cwp 3/25/2011 13:14'!
delete: aResolvable
	store delete: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'public' stamp: 'cwp 3/25/2011 19:14'!
delimiter
	"Return path delimiter used by this filesystem."

	^ store delimiter! !

!FileSystem methodsFor: 'public-enumerating' stamp: 'CamilloBruni 1/19/2012 00:32'!
directoriesAt: aResolvable
	^ Array streamContents: [ :out | 
		self directoriesAt: aResolvable do: [ :path|
			out nextPut: path ]].! !

!FileSystem methodsFor: 'public-enumerating' stamp: 'CamilloBruni 1/19/2012 00:30'!
directoriesAt: aResolvable do: aBlock
	| path |
	path := self resolve: aResolvable.
	store
		directoryAt: path
		ifAbsent: [ store signalDirectoryDoesNotExist: path ]
		directoryNodesDo: [ :entry | 
			aBlock value: path / (store basenameFromEntry: entry) ]! !

!FileSystem methodsFor: 'public-enumerating' stamp: 'CamilloBruni 7/10/2012 15:48'!
directoryNamesAt: aResolvable
	^ Array streamContents: [ :out | 
		self directoryNamesAt: aResolvable do: [ :name|
			out nextPut: name ]].! !

!FileSystem methodsFor: 'public-enumerating' stamp: 'CamilloBruni 7/10/2012 15:43'!
directoryNamesAt: aResolvable do: aBlock
	| path |
	path := self resolve: aResolvable.
	store
		directoryAt: path
		ifAbsent: [ store signalDirectoryDoesNotExist: path ]
		directoryNodesDo: [ :entry | 
			aBlock value: (store basenameFromEntry: entry) ]! !

!FileSystem methodsFor: 'public' stamp: 'cwp 3/25/2011 19:16'!
ensureDirectory: aResolvable
	"Resolve the argument to an absolute path, then ask the store to make
	sure that all the directories contained in the argument path exist or are created."
	
	store ensureDirectory: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'public-enumerating' stamp: 'GuillermoPolito 6/22/2012 12:21'!
entriesAt: aResolvable
	^ Array
		streamContents: [ :out | 
			self entriesAt: aResolvable do: [ :entry | out nextPut: entry ]
		]! !

!FileSystem methodsFor: 'public-enumerating' stamp: 'CamilloBruni 6/22/2012 17:57'!
entriesAt: aResolvable do: aBlock
	^ self 
		entriesAt: aResolvable 
		ifAbsent: [ store signalDirectoryDoesNotExist: ( self resolve: aResolvable) ]
		do: aBlock ! !

!FileSystem methodsFor: 'public-enumerating' stamp: 'CamilloBruni 6/22/2012 17:56'!
entriesAt: aResolvable ifAbsent: absentBlock do: aBlock
	| path entry aFilesystem |
	path := self resolve: aResolvable.
	aFilesystem := self.
	store
		directoryAt: path
		ifAbsent: [ ^ absentBlock value ]
		nodesDo: [ :node | 
			entry := store entryFromNode: node path: path for: aFilesystem.
			aBlock value: entry ]! !

!FileSystem methodsFor: 'public' stamp: 'EstebanLorenzano 4/3/2012 13:27'!
entryAt: aResolvable
	| path |
	path := self resolve: aResolvable.
	^ store
		nodeAt: path
		ifPresent: [ :node | store entryFromNode: node fileSystem: self path: path ]
		ifAbsent: [ store signalFileDoesNotExist: path ]! !

!FileSystem methodsFor: 'public' stamp: 'cwp 3/25/2011 19:19'!
exists: aResolvable
	"Resolve the argument, and answer true if the there is
	a file or directory at that path, false if there is not."
	
	^ store exists: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 5/7/2012 01:15'!
extensionDelimiter
	^ $.! !

!FileSystem methodsFor: 'public-enumerating' stamp: 'CamilloBruni 7/10/2012 15:44'!
fileNamesAt: aResolvable
	^ Array streamContents: [ :out | 
		self fileNamesAt: aResolvable do: [ :path|
			out nextPut: path ]].! !

!FileSystem methodsFor: 'public-enumerating' stamp: 'CamilloBruni 7/10/2012 15:44'!
fileNamesAt: aResolvable do: aBlock
	| path |
	path := self resolve: aResolvable.
	store
		directoryAt: path
		ifAbsent: [ store signalDirectoryDoesNotExist: path ]
		fileNodesDo: [ :entry | 
			aBlock value: (store basenameFromEntry: entry) ]! !

!FileSystem methodsFor: 'public-enumerating' stamp: 'CamilloBruni 1/19/2012 00:32'!
filesAt: aResolvable
	^ Array streamContents: [ :out | 
		self filesAt: aResolvable do: [ :path|
			out nextPut: path ]].! !

!FileSystem methodsFor: 'public-enumerating' stamp: 'CamilloBruni 1/19/2012 00:30'!
filesAt: aResolvable do: aBlock
	| path |
	path := self resolve: aResolvable.
	store
		directoryAt: path
		ifAbsent: [ store signalDirectoryDoesNotExist: path ]
		fileNodesDo: [ :entry | 
			aBlock value: path / (store basenameFromEntry: entry) ]! !

!FileSystem methodsFor: 'printing' stamp: 'cwp 2/18/2011 16:34'!
forReferencePrintOn: aStream
	store forReferencePrintOn: aStream! !

!FileSystem methodsFor: 'public-testing' stamp: 'StephaneDucasse 6/22/2012 17:30'!
hasChildren: aResolvable
	"Returns whether aResolvable has children."
	store directoryAt: (self resolve: aResolvable)
			ifAbsent: [ ^false ]
			nodesDo:  [ :node | ^true ].
	^false! !

!FileSystem methodsFor: 'public-testing' stamp: 'CamilloBruni 6/22/2012 17:57'!
hasDirectories: aResolvable
	self
		entriesAt: aResolvable
		ifAbsent: [  ^ false ]
		do: [ :entry | entry isDirectory ifTrue: [ ^true ] ].
	^false! !

!FileSystem methodsFor: 'public-testing' stamp: 'CamilloBruni 6/22/2012 17:57'!
hasFiles: aResolvable
	self
		entriesAt: aResolvable
		ifAbsent: [  ^ false ]
		do: [ :entry | entry isFile ifTrue: [ ^true ] ].
	^false! !

!FileSystem methodsFor: 'comparing' stamp: 'cwp 2/18/2011 16:08'!
hash
	^ store hash! !

!FileSystem methodsFor: 'initialize-release' stamp: 'cwp 2/18/2011 20:33'!
initializeWithStore: aStore
	store := aStore.
	workingDirectory := store defaultWorkingDirectory! !

!FileSystem methodsFor: 'public-testing' stamp: 'CamilloBruni 5/7/2012 01:19'!
isCaseSensitive
	^ self store isCaseSensitive! !

!FileSystem methodsFor: 'public-testing' stamp: 'cwp 3/25/2011 19:21'!
isDirectory: aResolvable
	"Resolve the argument, and answer true if the result refers
	to a directory, false if it refers to a file or doesn't exist."

	^ store isDirectory: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'public-testing' stamp: 'CamilloBruni 6/1/2012 14:45'!
isFile: aResolvable
	"Resolve the argument, and answer true if the result refers
	to a file, false if it refers to a directory or doesn't exist."

	^ store isFile: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'public-testing' stamp: 'CamilloBruni 7/10/2012 21:31'!
isReadable: aResolvable
	"Resolve the argument, and answer true if the there is
	a file or directory that can be read from."
	
	^ (self permissions: aResolvable) isReadable! !

!FileSystem methodsFor: 'public-testing' stamp: 'CamilloBruni 7/10/2012 21:31'!
isWritable: aResolvable
	"Resolve the argument, and answer true if the there is
	a file that can be written to or directory that can be changed."
	
	^  (self permissions: aResolvable) isWritable! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 5/9/2012 13:03'!
mimeTypesAt: aResolvable
	"Return the possible mime types for the given path."
	^ store mimeTypesAt: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 7/10/2012 20:54'!
modificationTime: aResolvable
	"Resolve the argument, and answer true if the result refers
	to a directory, false if it refers to a file or doesn't exist."

	^ store modificationTime: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'public' stamp: 'cwp 3/25/2011 19:23'!
open
	"Some kinds of filesystems need to open connections to external resources. Does nothing by default."
	
	store open! !

!FileSystem methodsFor: 'public' stamp: 'EstebanLorenzano 4/12/2012 14:25'!
open: aResolvable writable: aBoolean 
	"Resolve aResolvable into an absolute path, then ask the store to open the file at
	that path using the specified access mode."
	
	| path |
	path := self resolve: aResolvable.
	^ store handleClass 
		open: (FileReference fileSystem: self path: path) 
		writable: aBoolean
		! !

!FileSystem methodsFor: 'delegated' stamp: 'cwp 3/25/2011 13:16'!
openFileStream: aResolvable writable: aBoolean
	^ store 
		openFileStream: (self resolve: aResolvable) 
		writable: aBoolean! !

!FileSystem methodsFor: 'private' stamp: 'CamilloBruni 1/20/2012 13:34'!
openStreamDescription: aResolvable writable: aBoolean
	"I am  a helper method to delegate basicOpen:writable: to the store.
	 I am called from FSHandle implementations."
	
	| path |
	
	path := self resolve: aResolvable.
	^ store basicOpen: path writable: aBoolean! !

!FileSystem methodsFor: 'converting' stamp: 'cwp 11/21/2009 11:30'!
pathFromObject: anObject 
	^ anObject asPathWith: self! !

!FileSystem methodsFor: 'converting' stamp: 'cwp 2/18/2011 16:39'!
pathFromString: aString
	^ store pathFromString: aString! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 7/10/2012 21:31'!
permissions: aResolvable
	"Resolve the argument and return the Permissions for this file or directory "

	^ store permissions: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'printing' stamp: 'cwp 2/28/2011 12:29'!
printPath: aPath on: aStream
	store printPath: aPath on: aStream! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 1/20/2012 13:23'!
readStreamOn: aResolvable
	"Resolve the argument into an absolute path and open a file handle on the file
	at that path. Ask the handle to give us a read stream for reading the file."

	^ (self open: aResolvable writable: false) readStream.! !

!FileSystem methodsFor: 'public' stamp: 'EstebanLorenzano 4/12/2012 14:25'!
referenceTo: aResolvable 
	"Answer a reference to the argument from the context of the receiver filesystem.  	
		Example: FSFilesystem disk referenceTo: 'plonk.taz'"

	^ FileReference 
		fileSystem: self
		path: (self pathFromObject: aResolvable)! !

!FileSystem methodsFor: 'public' stamp: 'EstebanLorenzano 4/3/2012 13:24'!
rename: sourcePath ifAbsent: aBlock to: destPath ifPresent: pBlock
	"Rename the file referenced as sourcePath to the destination referred as destPath. 
	Perform associate actions in case of problems."
	
	| source destination |
	source := self resolve: sourcePath.
	destination := self resolve: destPath.
	store
		rename: source
		ifAbsent: aBlock
		to: destination
		ifPresent: pBlock
		fileSystem: self.
	^ destination! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 1/23/2012 15:49'!
rename: sourcePath to: destName
	"Rename the file referenced as sourcePath to destPath.  Raise exceptions 
	FileExists or FileDoesNotExist  if the operation fails"
	
	^ self
		rename: sourcePath
		ifAbsent: [store signalFileDoesNotExist: sourcePath]
		to: destName
		ifPresent: [self signalFileExists: destName]! !

!FileSystem methodsFor: 'public' stamp: 'cwp 3/29/2011 15:54'!
resolve: aResolvable
	^ aResolvable asResolvedBy: self! !

!FileSystem methodsFor: 'navigating' stamp: 'CamilloBruni 2/4/2012 19:22'!
resolvePath: aPath
	"Return a path where the argument is resolved in the context of the
	receiver. The behavior is similar to the one of a command line.
		> cd /a/b/c
		> cd b
		The shell will attempt to make /a/b/c/b the current directory. "

	^ workingDirectory resolve: aPath! !

!FileSystem methodsFor: 'navigating' stamp: 'EstebanLorenzano 4/3/2012 13:06'!
resolveReference: aReference
	^ aReference fileSystem = self ifTrue:
		[workingDirectory resolvePath: aReference path]! !

!FileSystem methodsFor: 'navigating' stamp: 'cwp 3/29/2011 15:56'!
resolveString: aString
	"Returns the root of the receiver filesystem, i.e. / on unix"
	
	^ workingDirectory resolvePath: (store pathFromString: aString)! !

!FileSystem methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
root
	"Returns the root of the receiver filesystem, i.e. / on unix"
	^ self referenceTo: Path root! !

!FileSystem methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 01:12'!
separator
	"Return path separator used by this filesystem."

	^ store separator! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 7/10/2012 22:09'!
size: aResolvable
	"Resolve the argument and return the size for this file or directory "

	^ store size: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'accessing' stamp: 'cwp 2/18/2011 16:08'!
store
	^ store! !

!FileSystem methodsFor: 'converting' stamp: 'cwp 2/18/2011 12:09'!
stringFromPath: aPath
	^ store stringFromPath: aPath! !

!FileSystem methodsFor: 'accessing' stamp: 'StephaneDucasse 7/3/2011 16:07'!
workingDirectory
	"Returns a reference to the directory from where the image was launched"

	^ self referenceTo: self workingDirectoryPath! !

!FileSystem methodsFor: 'accessing' stamp: 'cwp 3/29/2011 15:57'!
workingDirectoryPath
	^ workingDirectory! !

!FileSystem methodsFor: 'accessing' stamp: 'CamilloBruni 5/4/2012 19:14'!
workingDirectoryPath: aPath

	aPath isAbsolute 
		ifFalse: [ self error: 'Cannot set the working directory to a relative path' ].
	workingDirectory := aPath
! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 1/20/2012 13:23'!
writeStreamOn: aResolvable
	"Open a write stream on the file referred by the argument. It can be a string or a path"

	^ (self open: aResolvable writable: true) writeStream.! !

Object subclass: #FileSystemDirectoryEntry
	instanceVariableNames: 'reference creation modification isDirectory size posixPermissions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Public'!

!FileSystemDirectoryEntry commentStamp: 'cwp 11/18/2009 11:09' prior: 0!
I am a cache for metadata about a file or directory. The information I hold is as follows:

reference
	A reference to the file or directory to which my data pertains.
	
creation
	The creation date and time, stored as number seconds since the 
	Smalltalk epoch.
	
modification
	The modification date and time, number seconds since the Smalltalk epoch.
	
isDirectory 
	True if my data pertains to a directory, false if a file.
	
size
	Size in bytes for a file, 0 for a directory.
!

!FileSystemDirectoryEntry class methodsFor: 'accessing' stamp: 'EstebanLorenzano 6/22/2012 14:52'!
allPosixPermissions 
	^8r777! !

!FileSystemDirectoryEntry class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 6/22/2012 14:52'!
fileSystem: aFilesystem path: aPath creation: cTime modification: mTime isDir: aBoolean size: anInteger
	"Create a directory entry given a filesystem and a path in such filesystem. In addition, the creation and modification time are required as well as a boolean that indicates whether the entry is a folder or a file and its size."

	^ self 
		reference: (aFilesystem referenceTo: aPath)
		creation: cTime
		modification: mTime
		isDir: aBoolean
		size: anInteger
		posixPermissions: self allPosixPermissions! !

!FileSystemDirectoryEntry class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 6/22/2012 14:51'!
fileSystem: aFilesystem path: aPath creation: cTime modification: mTime isDir: aBoolean size: anInteger posixPermissions: posixNumber
	"Create a directory entry given a filesystem and a path in such filesystem. In addition, the creation and modification time are required as well as a boolean that indicates whether the entry is a folder or a file and its size."

	^ self 
		reference: (aFilesystem referenceTo: aPath)
		creation: cTime
		modification: mTime
		isDir: aBoolean
		size: anInteger
		posixPermissions: posixNumber! !

!FileSystemDirectoryEntry class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 6/22/2012 14:51'!
reference: ref creation: cTime modification: mTime isDir: aBoolean size: anInteger posixPermissions: posixNumber
	"Create a directory entry for the file reference ref, with the creation time, cTime, the modification time, mTime. aBoolean indicates if the entry represents a directory or a file of size given by anInteger"
	
	^ self basicNew
		initializeWithRef: ref
		creation: cTime
		modification: mTime
		isDir: aBoolean
		size: anInteger
		posixPermissions: posixNumber! !

!FileSystemDirectoryEntry methodsFor: 'converting' stamp: 'EstebanLorenzano 4/12/2012 14:28'!
asFileReference
	^ reference! !

!FileSystemDirectoryEntry methodsFor: 'converting' stamp: 'EstebanLorenzano 4/12/2012 14:30'!
asReference
	self deprecated: 'Use #asFileReference instead'  on: '12 April 2012' in:  'Pharo 1.4' .
	^ self asFileReference! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'StephaneDucasse 2/15/2010 17:59'!
basename

	^ reference basename! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'abc 5/11/2012 23:31'!
creation
	^ self creationTime ! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'StephaneDucasse 1/27/2011 22:16'!
creationSeconds
	"Return the creation date and time of the entry receiver in seconds."
		
	^ creation! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 21:20'!
creationTime
	"Return the creation date and time of the entry receiver."
	
	^ creation! !

!FileSystemDirectoryEntry methodsFor: 'delegate' stamp: 'CamilloBruni 8/15/2011 17:02'!
extension
	^ reference extension! !

!FileSystemDirectoryEntry methodsFor: 'delegate' stamp: 'CamilloBruni 8/12/2011 20:33'!
fullName
	^ reference fullName! !

!FileSystemDirectoryEntry methodsFor: 'initialize-release' stamp: 'EstebanLorenzano 6/22/2012 14:53'!
initializeWithRef: ref creation: cTime modification: mTime isDir: bool size: bytes posixPermissions: posixNumber 
	
	reference := ref.
	creation := cTime.
	modification := mTime.
	isDirectory := bool.
	size := bytes.
	posixPermissions := posixNumber! !

!FileSystemDirectoryEntry methodsFor: 'testing' stamp: 'StephaneDucasse 1/27/2011 22:16'!
isDirectory
	"Return whether the receiver is a directory"
	
	^ isDirectory! !

!FileSystemDirectoryEntry methodsFor: 'testing' stamp: 'StephaneDucasse 1/27/2011 22:16'!
isFile
	"Return whether the receiver is a file"
	
	^ isDirectory not! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'abc 5/11/2012 23:30'!
modification
	^ self modificationTime ! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'StephaneDucasse 1/27/2011 22:15'!
modificationSeconds
	"Return the modification date and time of the entry receiver in seconds."
	^ modification! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 21:20'!
modificationTime
	"Return the modification date and time of the entry receiver."

	^  modification! !

!FileSystemDirectoryEntry methodsFor: 'delegate' stamp: 'CamilloBruni 8/12/2011 20:56'!
pathSegments
	^ reference pathSegments! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'EstebanLorenzano 7/3/2012 11:29'!
permissions 
	^self posixPermissions
		ifNotNil: [ FileSystemPermission posixPermissions: self posixPermissions ]! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'EstebanLorenzano 6/22/2012 14:54'!
posixPermissions 
	^posixPermissions! !

!FileSystemDirectoryEntry methodsFor: 'printing' stamp: 'sd 2/11/2011 19:40'!
printOn: aStream

	aStream nextPutAll: 'DirectoryEntry: '.
	reference ifNotNilDo: [:ref | aStream nextPutAll: reference printString].! !

!FileSystemDirectoryEntry methodsFor: 'delegate' stamp: 'CamilloBruni 8/12/2011 20:32'!
readStream
	^ reference readStream! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'cwp 11/15/2009 21:54'!
reference
	^ reference! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'StephaneDucasse 1/27/2011 22:15'!
size
	"Returns the receiver size"
	^ size! !

!FileSystemDirectoryEntry methodsFor: 'delegate' stamp: 'CamilloBruni 8/12/2011 20:33'!
writeStream
	^ reference writeStream! !

Object subclass: #FileSystemGuide
	instanceVariableNames: 'visitor work'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!FileSystemGuide commentStamp: 'cwp 11/18/2009 12:09' prior: 0!
I am an abstract superclass for objects that fulfill the Guide role in the Guide/Visitor pattern. My subclasses know how to traverse a filesystem in a specific order, "showing" the files and directories they encounter to a visitor.

visitor
	An object that fulfills the Visitor role and implements the visitor protocol.
	
work
	An OrderedCollection, used to keep track of filesystem nodes that have not yet been visited!

FileSystemGuide subclass: #BreadthFirstGuide
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!BreadthFirstGuide commentStamp: 'cwp 11/18/2009 12:13' prior: 0!
I traverse the filesystem in breadth-first order. Given this hierarchy:

alpha
	beta
	gamma
delta
	epsilon

I would visit the nodes in the following order:  alpha, delta, beta, gamma, epsilon.

I use my work instance variable as a queue, adding nodes to be visited to the end and retrieving them from the beginning.
!

!BreadthFirstGuide methodsFor: 'showing' stamp: 'CamilloBruni 8/12/2011 18:23'!
show: aReference 
	self push: aReference entry.
	self whileNotDoneDo: [ self visitNextEntry: self top ]! !

!BreadthFirstGuide methodsFor: 'showing' stamp: 'CamilloBruni 8/12/2011 18:19'!
visitNextEntry: entry
	entry isFile 
		ifTrue: [ ^ visitor visitFile: entry ].
	
	visitor visitDirectory: entry.
	self pushAll: entry reference entries.! !

!FileSystemGuide class methodsFor: 'instance creation' stamp: 'cwp 10/29/2009 19:27'!
for: aVisitor
	^ self basicNew initializeWithVisitor: aVisitor! !

!FileSystemGuide class methodsFor: 'instance creation' stamp: 'cwp 11/17/2009 11:58'!
show: aReference to: aVisitor
	^ (self for: aVisitor) show: aReference! !

!FileSystemGuide methodsFor: 'initialize-release' stamp: 'StephaneDucasse 1/27/2011 10:37'!
initialize

	super initialize.
	work := OrderedCollection new! !

!FileSystemGuide methodsFor: 'initialize-release' stamp: 'cwp 10/29/2009 23:48'!
initializeWithVisitor: aVisitor
	self initialize.
	visitor := aVisitor.
	! !

!FileSystemGuide methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/12/2011 18:18'!
pop
	^ work removeLast! !

!FileSystemGuide methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/12/2011 18:18'!
push: anObject
	work add: anObject! !

!FileSystemGuide methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/12/2011 18:20'!
pushAll: aCollection 
	aCollection do: [ :ea | self push: ea ]! !

!FileSystemGuide methodsFor: 'showing' stamp: 'cwp 10/29/2009 23:21'!
show: aReference
	self subclassResponsibility! !

!FileSystemGuide methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/12/2011 18:22'!
top
	^ work removeFirst! !

!FileSystemGuide methodsFor: 'showing' stamp: 'lr 7/13/2010 15:36'!
whileNotDoneDo: aBlock
	[ work isEmpty ] whileFalse: [ aBlock value ]! !

FileSystemGuide subclass: #PostorderGuide
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!PostorderGuide commentStamp: 'cwp 11/18/2009 12:16' prior: 0!
I traverse the filesystem in depth-first post order. Given this hierarchy:

alpha
	beta
	gamma
delta
	epsilon

I would visit the nodes in the following order:  beta, gamma, alpha, epsilon, delta.

I use my work instance variable as a stack. I push messages that cause nodes to be traversed or visited, and execute them in reverse order.!

!PostorderGuide methodsFor: 'showing' stamp: 'CamilloBruni 8/12/2011 18:18'!
pushTraverse: aReference 
	self push: (Message 
				selector: #traverse:
				argument: aReference)! !

!PostorderGuide methodsFor: 'showing' stamp: 'CamilloBruni 8/12/2011 18:18'!
pushVisit: aReference 
	self push: (Message 
				selector: #visit:
				argument: aReference)! !

!PostorderGuide methodsFor: 'showing' stamp: 'cwp 11/16/2009 10:50'!
show: aReference 
	self pushTraverse: aReference entry.
	self whileNotDoneDo: [self pop sendTo: self ]! !

!PostorderGuide methodsFor: 'showing' stamp: 'cwp 11/16/2009 21:57'!
traverse: anEntry 
	self pushVisit: anEntry.
	anEntry isDirectory ifTrue: 
		[anEntry reference entries reverseDo: 
			[:ea | self pushTraverse: ea]]! !

!PostorderGuide methodsFor: 'showing' stamp: 'cwp 11/16/2009 10:50'!
visit: anEntry
	anEntry isDirectory
		ifTrue: [visitor visitDirectory: anEntry]
		ifFalse: [visitor visitFile: anEntry]
	! !

FileSystemGuide subclass: #PreorderGuide
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!PreorderGuide commentStamp: 'cwp 11/18/2009 12:18' prior: 0!
I traverse the filesystem in depth-first pre order. Given this hierarchy:

alpha
	beta
	gamma
delta
	epsilon

I would visit the nodes in the following order: alpha, beta, gamma, delta, epsilon.

I use my work instance variable as a stack. I push nodes to be visited and visit them in reverse order.!

!PreorderGuide methodsFor: 'showing' stamp: 'CamilloBruni 8/12/2011 18:19'!
pushAll: aCollection 
	aCollection reverseDo: [ :ea | work add: ea ]! !

!PreorderGuide methodsFor: 'showing' stamp: 'cwp 11/16/2009 10:47'!
show: aReference 
	self push: aReference entry.
	self whileNotDoneDo: 
		[| entry |
		entry := self pop.
		entry isFile 
			ifTrue: [visitor visitFile: entry]
			ifFalse: 
				[visitor visitDirectory: entry.
				self pushAll: entry reference entries]]! !

Object subclass: #FileSystemHandle
	instanceVariableNames: 'reference writable'
	classVariableNames: 'Primitives'
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!FileSystemHandle commentStamp: 'cwp 11/18/2009 11:11' prior: 0!
I am an abstract superclass for file handle implementations. I provide a uniform interface that streams can use for read and write operations on a file regardless of the filesystem. I encapsulate the actual IO primitives.!

!FileSystemHandle class methodsFor: 'instance creation' stamp: 'cwp 7/26/2009 12:52'!
on: aReference writable: aBoolean
	^ self new setReference: aReference writable: aBoolean! !

!FileSystemHandle class methodsFor: 'instance creation' stamp: 'cwp 7/26/2009 12:52'!
open: aReference writable: aBoolean
	^ (self on: aReference writable: aBoolean) open! !

!FileSystemHandle methodsFor: 'public' stamp: 'cwp 7/29/2009 22:19'!
at: index
	| buffer |
	buffer := ByteArray new: 1.
	self at: index read: buffer startingAt: 1 count: 1.
	^ buffer at: 1! !

!FileSystemHandle methodsFor: 'public' stamp: 'lr 4/13/2010 16:10'!
at: index put: anObject
	| buffer |
	buffer := ByteArray with: (anObject isCharacter
		ifTrue: [ anObject codePoint ]
		ifFalse: [ anObject ]).
	self at: index write: buffer startingAt: 1 count: 1.
	! !

!FileSystemHandle methodsFor: 'public' stamp: 'cwp 7/26/2009 12:50'!
at: offset read: buffer startingAt: start count: count
	self subclassResponsibility! !

!FileSystemHandle methodsFor: 'public' stamp: 'cwp 7/26/2009 12:50'!
at: offset write: buffer startingAt: start count: count
	self subclassResponsibility! !

!FileSystemHandle methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:06'!
basicOpen
	"get the raw stream description from the filesystem's store"
	^ self fileSystem 
		openStreamDescription: reference path
		writable: writable! !

!FileSystemHandle methodsFor: 'public' stamp: 'cwp 7/26/2009 12:50'!
close
	self subclassResponsibility! !

!FileSystemHandle methodsFor: 'public' stamp: 'SeanDeNigris 6/13/2012 07:47'!
ensureClosed
	reference exists ifTrue: [self close]! !

!FileSystemHandle methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/3/2012 13:02'!
fileSystem
	^ reference fileSystem ! !

!FileSystemHandle methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/3/2012 13:03'!
filesystem
	self deprecated: 'Use fileSystem' on: '3 April 2012' in: 'Pharo 1.4'.
	^ self fileSystem ! !

!FileSystemHandle methodsFor: 'public' stamp: 'cwp 7/26/2009 12:50'!
flush
	self subclassResponsibility! !

!FileSystemHandle methodsFor: 'accessing' stamp: 'abc 5/11/2012 23:24'!
fullName
	^ reference fullName! !

!FileSystemHandle methodsFor: 'testing' stamp: 'cwp 7/26/2009 12:50'!
isOpen
	self subclassResponsibility! !

!FileSystemHandle methodsFor: 'testing' stamp: 'abc 5/11/2012 23:24'!
isWritable
	^ writable! !

!FileSystemHandle methodsFor: 'public' stamp: 'cwp 7/26/2009 12:50'!
open
	self subclassResponsibility! !

!FileSystemHandle methodsFor: 'public' stamp: 'CamilloBruni 6/22/2012 21:51'!
readStream
	self subclassResponsability! !

!FileSystemHandle methodsFor: 'accessing' stamp: 'cwp 7/26/2009 12:51'!
reference
	^ reference! !

!FileSystemHandle methodsFor: 'public' stamp: 'cwp 7/26/2009 12:51'!
reopen
	self close.
	self open! !

!FileSystemHandle methodsFor: 'initialize-release' stamp: 'cwp 11/20/2009 14:56'!
setReference: aReference writable: aBoolean
	reference := aReference resolve.
	writable := aBoolean! !

!FileSystemHandle methodsFor: 'public' stamp: 'cwp 7/26/2009 12:50'!
truncateTo: anInteger
	self subclassResponsibility! !

!FileSystemHandle methodsFor: 'public' stamp: 'CamilloBruni 6/22/2012 21:52'!
writeStream
	self subclassResponsability! !

Object subclass: #FileSystemPermission
	instanceVariableNames: 'posixPermission'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Public'!

!FileSystemPermission commentStamp: '<historical>' prior: 0!
I'm a set of permissions for a Directory Entry!

!FileSystemPermission class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 6/22/2012 15:02'!
new 
	self error: 'Should not be called. Use #posixPermission: instead'! !

!FileSystemPermission class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 6/22/2012 15:01'!
posixPermissions: aNumber
	^self basicNew 
		initialize: aNumber;
		yourself! !

!FileSystemPermission methodsFor: 'comparing' stamp: 'CamilloBruni 7/10/2012 22:01'!
< other
	^ posixPermission < other posixPermission! !

!FileSystemPermission methodsFor: 'accessing' stamp: 'EstebanLorenzano 6/22/2012 15:10'!
groupExecute
	^ self permissionBitAt: 4! !

!FileSystemPermission methodsFor: 'accessing' stamp: 'EstebanLorenzano 6/22/2012 15:10'!
groupRead
	^ self permissionBitAt: 6! !

!FileSystemPermission methodsFor: 'accessing' stamp: 'EstebanLorenzano 6/22/2012 15:11'!
groupWrite
	^ self permissionBitAt: 5! !

!FileSystemPermission methodsFor: 'initialization' stamp: 'EstebanLorenzano 6/22/2012 15:02'!
initialize: aNumber
	posixPermission := aNumber.
	self initialize.	! !

!FileSystemPermission methodsFor: 'testing' stamp: 'CamilloBruni 7/10/2012 21:32'!
isReadable
	^ self ownerRead! !

!FileSystemPermission methodsFor: 'testing' stamp: 'CamilloBruni 7/10/2012 21:33'!
isWritable
	^ self ownerWrite! !

!FileSystemPermission methodsFor: 'accessing' stamp: 'EstebanLorenzano 6/22/2012 15:11'!
otherExecute
	^ self permissionBitAt: 1! !

!FileSystemPermission methodsFor: 'accessing' stamp: 'EstebanLorenzano 6/22/2012 15:11'!
otherRead
	^ self permissionBitAt: 3! !

!FileSystemPermission methodsFor: 'accessing' stamp: 'EstebanLorenzano 6/22/2012 15:11'!
otherWrite
	^ self permissionBitAt: 2! !

!FileSystemPermission methodsFor: 'accessing' stamp: 'EstebanLorenzano 6/22/2012 15:12'!
ownerExecute
	^ self permissionBitAt: 7! !

!FileSystemPermission methodsFor: 'accessing' stamp: 'EstebanLorenzano 6/22/2012 15:12'!
ownerRead
	^ self permissionBitAt: 9! !

!FileSystemPermission methodsFor: 'accessing' stamp: 'EstebanLorenzano 6/22/2012 15:12'!
ownerWrite
	^ self permissionBitAt: 8! !

!FileSystemPermission methodsFor: 'accessing' stamp: 'EstebanLorenzano 7/3/2012 11:28'!
permissionBitAt: bitIndex
	^ (posixPermission bitAt: bitIndex) == 1! !

!FileSystemPermission methodsFor: 'printing' stamp: 'EstebanLorenzano 6/22/2012 15:06'!
printOn: aStream
	aStream 
		"Owner"
		nextPut: (self ownerRead ifTrue: [ $r ] ifFalse: [ $- ]);
		nextPut: (self ownerWrite ifTrue: [ $w ] ifFalse: [ $- ]);
		nextPut: (self ownerExecute ifTrue: [ $x ] ifFalse: [ $- ]);
		"Group"
		nextPut: (self groupRead ifTrue: [ $r ] ifFalse: [ $- ]);
		nextPut: (self groupWrite ifTrue: [ $w ] ifFalse: [ $- ]);
		nextPut: (self groupExecute ifTrue: [ $x ] ifFalse: [ $- ]);
		"Other"
		nextPut: (self otherRead ifTrue: [ $r ] ifFalse: [ $- ]);
		nextPut: (self otherWrite ifTrue: [ $w ] ifFalse: [ $- ]);
		nextPut: (self otherExecute ifTrue: [ $x ] ifFalse: [ $- ])
		! !

Object subclass: #FileSystemResolver
	instanceVariableNames: 'next'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!FileSystemResolver commentStamp: 'cwp 3/29/2011 17:04' prior: 0!
I am an abstract superclass for objects that can resolve origins into references. Such objects use the Chain of Responsibility pattern, and when unable to resolve a particular origin, delegate that resolution request to the next resolver in the list.

next
	The next resolver in the list, or nil
!

!FileSystemResolver methodsFor: 'accessing' stamp: 'cwp 10/26/2009 20:53'!
addResolver: aResolver
	next
		ifNil: [next := aResolver]
		ifNotNil: [next addResolver: aResolver]! !

!FileSystemResolver methodsFor: 'resolving' stamp: 'cwp 10/27/2009 11:18'!
canResolve: aSymbol
	^ self supportedOrigins includes: aSymbol! !

!FileSystemResolver methodsFor: 'accessing' stamp: 'cwp 10/27/2009 10:28'!
flushCaches
	self flushLocalCache.
	next ifNotNil: [next flushCaches]! !

!FileSystemResolver methodsFor: 'accessing' stamp: 'cwp 10/27/2009 10:28'!
flushLocalCache! !

!FileSystemResolver methodsFor: 'accessing' stamp: 'cwp 10/27/2009 11:25'!
next
	^ next! !

!FileSystemResolver methodsFor: 'resolving' stamp: 'cwp 10/27/2009 11:18'!
resolve: aSymbol
	^ (self canResolve: aSymbol)
		ifTrue: [self perform: aSymbol]
		ifFalse: [self unknownOrigin: aSymbol]! !

!FileSystemResolver methodsFor: 'resolving' stamp: 'EstebanLorenzano 4/12/2012 14:25'!
resolveString: aString 
	| decoded fs |
	decoded := (FilePath pathName: aString isEncoded: true) asSqueakPathName.
	fs := FileSystem disk.
	^ FileReference
		fileSystem: fs
		path: (fs pathFromString: decoded)! !

!FileSystemResolver methodsFor: 'resolving' stamp: 'cwp 10/26/2009 20:06'!
supportedOrigins
	^ #()! !

!FileSystemResolver methodsFor: 'resolving' stamp: 'cwp 10/27/2009 09:26'!
unknownOrigin: aSymbol
	^ next ifNotNil: [next resolve: aSymbol]! !

FileSystemResolver subclass: #InteractiveResolver
	instanceVariableNames: 'cache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!InteractiveResolver commentStamp: 'cwp 11/18/2009 11:56' prior: 0!
I resolve origins by consulting the user. I maintain a cache of the user's responses.!

!InteractiveResolver methodsFor: 'accessing' stamp: 'cwp 10/27/2009 10:29'!
flushLocalCache
	cache := IdentityDictionary new! !

!InteractiveResolver methodsFor: 'initialize-release' stamp: 'cwp 10/27/2009 10:29'!
initialize
	self flushLocalCache! !

!InteractiveResolver methodsFor: 'resolving' stamp: 'cwp 10/27/2009 10:12'!
resolve: origin
	^ cache at: origin ifAbsent: [self unknownOrigin: origin]
	
! !

!InteractiveResolver methodsFor: 'resolving' stamp: 'EstebanLorenzano 4/2/2012 11:38'!
unknownOrigin: origin
	| reference |
	^ (next ifNotNil: [next resolve: origin]) ifNil: 
			[reference := ResolutionRequest for: origin.
			reference ifNotNil: [cache at: origin put: reference]]! !

FileSystemResolver subclass: #PlatformResolver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!PlatformResolver commentStamp: 'cwp 11/18/2009 11:56' prior: 0!
I am an abstract superclass for platform-specific resolvers.!

PlatformResolver subclass: #MacOSResolver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!MacOSResolver commentStamp: 'cwp 11/18/2009 11:57' prior: 0!
I am an expert on Mac OS X filesystem conventions. I resolve origins according to these conventions.!

!MacOSResolver class methodsFor: 'accessing' stamp: 'cwp 10/27/2009 10:59'!
platformName
	^  'Mac OS'! !

!MacOSResolver methodsFor: 'origins' stamp: 'cwp 10/27/2009 21:45'!
desktop
	^ self home / 'Desktop'! !

!MacOSResolver methodsFor: 'origins' stamp: 'cwp 10/27/2009 21:50'!
documents
	^ self home / 'Documents'! !

!MacOSResolver methodsFor: 'origins' stamp: 'StephaneDucasse 7/5/2011 16:38'!
home
	^ (self resolveString: self class primitiveGetUntrustedUserDirectory)
		parent parent parent parent parent! !

!MacOSResolver methodsFor: 'origins' stamp: 'CamilloBruni 5/24/2012 12:11'!
library
	^ self home / 'Library' ! !

!MacOSResolver methodsFor: 'origins' stamp: 'CamilloBruni 5/24/2012 12:46'!
preferences 
	^ self library / 'Preferences'! !

!MacOSResolver methodsFor: 'resolving' stamp: 'CamilloBruni 5/24/2012 14:51'!
supportedOrigins
	^ super supportedOrigins , #( library )! !

!PlatformResolver class methodsFor: 'instance creation' stamp: 'tg 11/8/2010 19:05'!
forCurrentPlatform
	| platformName |
	platformName :=  Smalltalk os platformName.
	^ (self allSubclasses detect: [:ea | ea platformName = platformName]) new! !

!PlatformResolver class methodsFor: 'accessing' stamp: 'cwp 10/27/2009 10:58'!
platformName
	^ nil! !

!PlatformResolver class methodsFor: 'private' stamp: 'CamilloBruni 8/12/2011 17:36'!
primitiveGetUntrustedUserDirectory
	<primitive: 'primitiveGetUntrustedUserDirectory' module: 'SecurityPlugin'>
	self primitiveFailed.
	self flag: 'use a more decent way to get a path in the users home directory'.! !

!PlatformResolver methodsFor: 'origins' stamp: 'lr 7/13/2010 15:35'!
desktop
	^ self subclassResponsibility! !

!PlatformResolver methodsFor: 'origins' stamp: 'lr 7/13/2010 15:35'!
documents
	^ self subclassResponsibility! !

!PlatformResolver methodsFor: 'origins' stamp: 'cwp 10/27/2009 21:01'!
home
	^ self subclassResponsibility! !

!PlatformResolver methodsFor: 'origins' stamp: 'CamilloBruni 5/24/2012 12:07'!
preferences
	^ self subclassResponsibility! !

!PlatformResolver methodsFor: 'resolving' stamp: 'CamilloBruni 5/24/2012 12:46'!
supportedOrigins
	^ #(home desktop documents preferences)! !

PlatformResolver subclass: #UnixResolver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!UnixResolver commentStamp: '<historical>' prior: 0!
I am an expert on Unix filesystem conventions. I resolve origins according to these conventions.!

!UnixResolver class methodsFor: 'accessing' stamp: 'pls
12/18/2009 04:53'!
platformName
	^  'unix'! !

!UnixResolver methodsFor: 'origins' stamp: 'pls 12/18/2009
04:52'!
desktop
	^ self home / 'Desktop'! !

!UnixResolver methodsFor: 'origins' stamp: 'pls 12/18/2009
04:53'!
documents
	^ self home / 'Documents'! !

!UnixResolver methodsFor: 'origins' stamp: 'CamilloBruni 1/19/2012 00:55'!
home
	"FIXME: hackish solution. should use env vars"
	^ (self resolveString: self class primitiveGetUntrustedUserDirectory)
		parent parent parent ! !

!UnixResolver methodsFor: 'origins' stamp: 'CamilloBruni 5/24/2012 12:12'!
preferences
	^ self home / '.config' ! !

PlatformResolver subclass: #WindowsResolver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!WindowsResolver commentStamp: 'cwp 11/18/2009 11:57' prior: 0!
I am an expert on Windows filesystem conventions. I resolve origins according to these conventions.!

!WindowsResolver class methodsFor: 'accessing' stamp: 'cwp 10/27/2009 10:59'!
platformName
	^ 'Win32'! !

!WindowsResolver methodsFor: 'origins' stamp: 'cwp 10/27/2009 21:45'!
desktop
	^ self home / 'Desktop'! !

!WindowsResolver methodsFor: 'origins' stamp: 'cwp 10/27/2009 21:50'!
documents
	^ self home / 'My Documents'! !

!WindowsResolver methodsFor: 'origins' stamp: 'CamilloBruni 8/12/2011 17:17'!
home
	| pathString |
	pathString := self class primitiveGetUntrustedUserDirectory.
	^ (self resolveString: pathString) parent parent! !

!WindowsResolver methodsFor: 'origins' stamp: 'EstebanLorenzano 6/21/2012 09:42'!
preferences
	"For further details refer to http://code.google.com/p/pharo/issues/detail?id=5255" 

	|  osMajorVer osMinorVer rootPath exeFolderList  |
	
	self flag: #todo. "This is still not working (and is crap). It does not work with localized windows and other stuff. Only razonable way to do this is to create a plugin who access right preferences"
	
	"First try to adhere to hardcoded Microsoft conventions - though we really should be reading environment variables since %SYSTEMDRIVE% might not be C: "
	
       osMajorVer := Smalltalk os osVersion first.   
       osMinorVer := Smalltalk os osVersion third.

       osMajorVer >= $6 
		ifTrue: [ rootPath := 'C:\ProgramData\Pharo\' ].   "Vista, 2008, 7 and later"  
       osMajorVer = $5 
		ifTrue: [ rootPath := 'C:\Documents and Settings\All Users\Application Data\Pharo\' ]. "2000, XP, 2003"
       osMajorVer = $4 
		ifTrue: [  rootPath := 'C:\Windows\All Users\Application Data\Pharo\' . "98, 98SE, Me"
           osMinorVer = $0 
			ifTrue: [ rootPath := 'C:\WinNT\Profiles\All Users\Application Data\Pharo\' ] .    "NT 4"
     	].
       osMajorVer = $3 
		ifTrue: [ rootPath := 'C:\WinNT\Profiles\All Users\Application Data\Pharo\' ] . "NT 3.1 3.5 3.51"	

	"Now work out which rootPath user is using.  Perhaps we should inform users of their specific options"	
	rootPath ifNotNil: [ 
		rootPath := rootPath asFileReference.
		rootPath exists 
			 ifTrue: [ ^ rootPath ] ].
		
	"This "	
	rootPath := FileSystem disk workingDirectory / '_config' / 'Pharo'.
	(rootPath exists and: [ rootPath isDirectory ])
		ifTrue: [ ^ rootPath ].
		
	^ FileSystem disk workingDirectory. "Avoids later MNU"! !

FileSystemResolver subclass: #SystemResolver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!SystemResolver commentStamp: 'cwp 11/18/2009 11:58' prior: 0!
I resolve origins that are related to the currently running Smalltalk system, using primitives provided by the VM. !

!SystemResolver methodsFor: 'origins' stamp: 'CamilloBruni 7/10/2012 21:50'!
changes
	^ self image withExtension: Smalltalk changesSuffix! !

!SystemResolver methodsFor: 'origins' stamp: 'cwp 10/26/2009 20:04'!
image
	^ self resolveString: self primImagePath! !

!SystemResolver methodsFor: 'origins' stamp: 'lr 7/13/2010 13:34'!
imageDirectory
	^ self image parent! !

!SystemResolver methodsFor: 'primitives' stamp: 'cwp 10/26/2009 20:05'!
primImagePath
	"Answer the full path name for the current image."

	<primitive: 121>
	self primitiveFailed! !

!SystemResolver methodsFor: 'primitives' stamp: 'lr 7/13/2010 13:27'!
primVmDirectoryPath
	"Answer the full path name for the current virtual machine."

	<primitive: 142>
	self primitiveFailed! !

!SystemResolver methodsFor: 'resolving' stamp: 'CamilloBruni 7/10/2012 21:34'!
supportedOrigins
	^ #(image imageDirectory changes vmBinary vmDirectory workingDirectory)! !

!SystemResolver methodsFor: 'origins' stamp: 'MarcusDenker 7/10/2012 10:39'!
vmBinary
	^ self resolveString: (Smalltalk vm vmFullPath)! !

!SystemResolver methodsFor: 'origins' stamp: 'lr 7/13/2010 13:33'!
vmDirectory
	^ self resolveString: self primVmDirectoryPath! !

Object subclass: #FileSystemStore
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!FileSystemStore commentStamp: '<historical>' prior: 0!
I am an abstract superclass for store implementations. My subclasses provide access to the actual data storage of a particular kind of filesystem. 

The file system can be accessed via
	FileSystem disk 
	FileSystem memory
	
My associated filesystem can be accessed as follows:
      DiskStore currentFileSystem!

!FileSystemStore class methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 01:10'!
delimiter
	self shouldBeImplemented ! !

!FileSystemStore class methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 01:20'!
isCaseSensitive
	^ self shouldBeImplemented! !

!FileSystemStore class methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 01:11'!
separator
	self shouldBeImplemented! !

!FileSystemStore methodsFor: 'public' stamp: 'CamilloBruni 6/22/2012 21:59'!
basenameFromEntry: aNode
	"Used to extract the basename from the low-level representation (node / entry) from the store."
	self subclassResponsability! !

!FileSystemStore methodsFor: 'abstract' stamp: 'CamilloBruni 7/10/2012 20:56'!
basicCreationTime: aNode
	"Used to decide whether the low-level representation (node / entry) from the store is a readable
	file or a directory whose contents can be listed."
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'abstract' stamp: 'CamilloBruni 1/20/2012 11:34'!
basicIsDirectory: aNode
	"Used to decide whether the low-level representation (node / entry) from the store is a directory.
	This private message should only be called form within the store."
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'abstract' stamp: 'CamilloBruni 1/20/2012 11:34'!
basicIsFile: aNode
	"Used to decide whether the low-level representation (node / entry) from the store is a file.
	This private message should only be called form within the store."
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'abstract' stamp: 'CamilloBruni 5/9/2012 00:41'!
basicIsWritable: aNode
	"Used to decide whether the low-level representation (node / entry) from the store is 
	a writable file or can be changed."
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'abstract' stamp: 'CamilloBruni 7/10/2012 20:56'!
basicModificationTime: aNode
	"Used to decide whether the low-level representation (node / entry) from the store is a readable
	file or a directory whose contents can be listed."
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'error signalling' stamp: 'CamilloBruni 1/20/2012 13:34'!
basicOpen: aPath writable: aBoolean
	"open the file at the given path and return an identifier"
	self subclassResponsibility! !

!FileSystemStore methodsFor: 'abstract' stamp: 'CamilloBruni 7/10/2012 21:15'!
basicPosixPermissions: aNode
	"Used to get the posix permissions from a low-level filesystem entry / node"
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'abstract' stamp: 'CamilloBruni 7/10/2012 21:12'!
basicSize: aNode
	"Used to get the size of the low-level representation (node / entry) "
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'public' stamp: 'CamilloBruni 5/10/2012 16:01'!
checkName: aString fixErrors: fixErrors
	^ self subclassResponsibility! !

!FileSystemStore methodsFor: 'abstract' stamp: 'cwp 2/19/2011 01:39'!
close
	"Some kinds of filesystems need to open connections to external resources"! !

!FileSystemStore methodsFor: 'private' stamp: 'CamilloBruni 6/22/2012 20:07'!
copy: sourcePath ifAbsent: absentBlock to: destinationPath ifPresent: presentBlock fileSystem: aFilesystem

	| buffer out in |
	
	in := nil.
	out := nil.
	buffer := nil.
	[
		in := aFilesystem readStreamOn: sourcePath.
		in ifNil: [ ^ absentBlock value ].
		
		(self exists: destinationPath)
			ifTrue: [ "cannot overwrite destination"
				^ presentBlock value ].
			
		out := aFilesystem writeStreamOn: destinationPath.
		buffer := ByteArray new: 1024.
		
		[ in atEnd ]
			whileFalse: [ 
				buffer := in nextInto: buffer.
				out nextPutAll: buffer ]]
	ensure: [ 
		in ifNotNil: [ in close ].
		out ifNotNil: [ out close ]]! !

!FileSystemStore methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:28'!
copy: sourcePath ifAbsent: absentBlock to: destinationPath ifPresent: presentBlock filesystem: aFilesystem
	self deprecated: 'Use #copy:ifAbsent:to:ifPresent:fileSystem:' on: '3 April 2012' in: 'Pharo 1.4'.
	^self 
		copy: sourcePath 
		ifAbsent: absentBlock 
		to: destinationPath 
		ifPresent: presentBlock 
		fileSystem: aFilesystem! !

!FileSystemStore methodsFor: 'abstract' stamp: 'cwp 2/18/2011 23:59'!
createDirectory: aPath
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'public' stamp: 'CamilloBruni 7/10/2012 20:56'!
creationTime: aPath
	self 
		nodeAt: aPath 
		ifPresent: [ :entry | ^ self basicCreationTime: entry ] 
		ifAbsent: [ ^ false ].
	! !

!FileSystemStore methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
defaultWorkingDirectory
	^ Path root! !

!FileSystemStore methodsFor: 'abstract' stamp: 'cwp 2/18/2011 13:25'!
delete: aPath
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'accessing' stamp: 'cwp 2/18/2011 17:19'!
delimiter
	^ self class delimiter! !

!FileSystemStore methodsFor: 'abstract' stamp: 'CamilloBruni 1/19/2012 00:15'!
directoryAt: aPath ifAbsent: absentBlock directoryNodesDo: aBlock
	^ self 
		directoryAt: aPath 
		ifAbsent: absentBlock 
		nodesDo: [ :entry|
			(self basicIsDirectory: entry) 
				ifTrue: [ aBlock value: entry ]].! !

!FileSystemStore methodsFor: 'abstract' stamp: 'CamilloBruni 7/10/2012 21:05'!
directoryAt: aPath ifAbsent: absentBlock fileNodesDo: aBlock
	^ self 
		directoryAt: aPath 
		ifAbsent: absentBlock 
		nodesDo: [ :entry|
			(self basicIsDirectory: entry) 
				ifFalse: [ aBlock value: entry ]].! !

!FileSystemStore methodsFor: 'abstract' stamp: 'cwp 2/18/2011 13:07'!
directoryAt: aPath ifAbsent: absentBlock nodesDo: aBlock
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'public' stamp: 'cwp 2/19/2011 00:00'!
ensureDirectory: aPath
	(self isDirectory: aPath) ifTrue: [ ^ self ].
	self ensureDirectory: aPath parent.
	self createDirectory: aPath! !

!FileSystemStore methodsFor: 'private' stamp: 'CamilloBruni 7/10/2012 21:05'!
entryFromNode: aNode fileSystem: aFilesystem path: aPath
	^ FileSystemDirectoryEntry
		fileSystem: aFilesystem
		path: aPath
		creation: (self basicCreationTime: aNode)
		modification:(self basicModificationTime: aNode)
		isDir: (self basicIsDirectory: aNode)
		size: (self basicSize: aNode)
		posixPermissions: (self basicPosixPermissions: aNode)! !

!FileSystemStore methodsFor: 'public' stamp: 'CamilloBruni 6/22/2012 21:02'!
entryFromNode: node path: path for: aFileystem
	| entryPath |
	entryPath := path / (self basenameFromEntry: node).
	^ self entryFromNode: node fileSystem: aFileystem path: entryPath! !

!FileSystemStore methodsFor: 'public' stamp: 'cwp 2/18/2011 13:11'!
exists: aPath
	self nodeAt: aPath 
		ifPresent: [ :entry | ^ true ] 
		ifAbsent: [ ^ false ].
	! !

!FileSystemStore methodsFor: 'private' stamp: 'cwp 2/18/2011 12:28'!
filename: aByteString matches: aByteString2
	^ aByteString = aByteString2! !

!FileSystemStore methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 01:20'!
isCaseSensitive
	^ self class isCaseSensitive! !

!FileSystemStore methodsFor: 'public' stamp: 'cwp 2/18/2011 13:22'!
isDirectory: aPath
	aPath isRoot ifTrue: [ ^ true ].
	self 
		nodeAt: aPath 
		ifPresent: [ :entry | ^ self basicIsDirectory: entry ] 
		ifAbsent: [ ^ false ].
	! !

!FileSystemStore methodsFor: 'public' stamp: 'CamilloBruni 1/20/2012 13:13'!
isFile: aPath
	"slow solution for big directories!! "
	^ self 
		nodeAt: aPath 
		ifPresent: [ :entry | ^ self basicIsFile: entry ] 
		ifAbsent: [ ^ false ]
	! !

!FileSystemStore methodsFor: 'public' stamp: 'CamilloBruni 5/9/2012 00:40'!
isWritable: aPath
	self nodeAt: aPath 
		ifPresent: [ :entry | ^ self basicIsWritable: entry ] 
		ifAbsent: [ ^ false ].
	! !

!FileSystemStore methodsFor: 'public' stamp: 'CamilloBruni 5/9/2012 13:06'!
mimeTypesAt: aPath
	^ MIMEType forExtensionReturnMimeTypesOrNil: aPath extension! !

!FileSystemStore methodsFor: 'public' stamp: 'CamilloBruni 7/10/2012 20:56'!
modificationTime: aPath
	self 
		nodeAt: aPath 
		ifPresent: [ :entry | ^ self basicModificationTime: entry ] 
		ifAbsent: [ ^ false ].
	! !

!FileSystemStore methodsFor: 'abstract' stamp: 'cwp 2/18/2011 12:50'!
nodeAt: aPath ifPresent: presentBlock ifAbsent: absentBlock
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'abstract' stamp: 'cwp 2/19/2011 01:38'!
open
	"Some kinds of filesystems need to open connections to external resources"! !

!FileSystemStore methodsFor: 'public' stamp: 'CamilloBruni 6/22/2012 21:03'!
openFileStream: path writable: writable
	self subclassResponsability! !

!FileSystemStore methodsFor: 'converting' stamp: 'CamilloBruni 5/7/2012 02:20'!
pathFromString: aString
	"Use the unix convention by default, since many filesystems are based on it."
	
	^ Path from: aString delimiter: self delimiter! !

!FileSystemStore methodsFor: 'public' stamp: 'PavelKrivanek 7/12/2012 13:22'!
permissions: aPath
	self nodeAt: aPath 
		ifPresent: [ :entry | ^ FileSystemPermission posixPermissions: (self basicPosixPermissions:  entry) ] 
		ifAbsent: [ ^ false ].
	! !

!FileSystemStore methodsFor: 'converting' stamp: 'cwp 2/28/2011 12:28'!
printPath: aPath on: out
	"Use the unix convention by default, since it's the most common."
	
	aPath isAbsolute ifTrue: [ out nextPut: $/ ].
	^ aPath printOn: out delimiter: self delimiter! !

!FileSystemStore methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:19'!
rename: sourcePath ifAbsent: absentBlock to: destinationPath ifPresent: presentBlock fileSystem: anFSFilesystem

	| result |
	result := self rename: sourcePath to: destinationPath.
	result	ifNil: [
		(self exists: destinationPath) 
			ifTrue: [ ^ presentBlock value ].
		(self exists: sourcePath) 
			ifFalse: [ ^ absentBlock value ].
		self primitiveFailed ].
	^ self.! !

!FileSystemStore methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:20'!
rename: sourcePath ifAbsent: absentBlock to: destinationPath ifPresent: presentBlock filesystem: anFSFilesystem

	self deprecated: 'Use #rename:ifAbsent:to:ifPresent:filesystem:' on: '3 April 2012' in: 'Pharo 1.4'.
	^ self 
		rename: sourcePath 
		ifAbsent: absentBlock 
		to: destinationPath 
		ifPresent: presentBlock 
		fileSystem: anFSFilesystem.! !

!FileSystemStore methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 01:13'!
separator
	^ self class separator! !

!FileSystemStore methodsFor: 'error signalling' stamp: 'EstebanLorenzano 4/2/2012 11:41'!
signalDirectoryDoesNotExist: aPath
	"Raise FSDirectoryDoesNotExist exception"
		
	^ DirectoryDoesNotExist signalWith: aPath! !

!FileSystemStore methodsFor: 'error signalling' stamp: 'EstebanLorenzano 4/2/2012 11:41'!
signalDirectoryExists: aPath
	"Raise FSDirectoryExists exception"
		
	^ DirectoryExists signalWith: aPath! !

!FileSystemStore methodsFor: 'error signalling' stamp: 'EstebanLorenzano 4/2/2012 11:41'!
signalFileDoesNotExist: aPath
	"Raise FSFileDoesNotExist exception"
		
	^ FileDoesNotExist signalWith: aPath! !

!FileSystemStore methodsFor: 'error signalling' stamp: 'EstebanLorenzano 4/2/2012 11:41'!
signalFileExists: aPath
	"Raise FSFileExist exception"
	
	^ FileExists signalWith: aPath! !

!FileSystemStore methodsFor: 'public' stamp: 'CamilloBruni 7/10/2012 22:08'!
size: aPath
	^ self 
		nodeAt: aPath 
		ifPresent: [ :entry | ^ self basicSize: entry ] 
		ifAbsent: [ ^ false ]
	! !

!FileSystemStore methodsFor: 'converting' stamp: 'CamilloBruni 1/19/2012 00:24'!
stringFromPath: aPath
	^ String streamContents: [ :out | 
		self printPath: aPath on: out ]! !

Object subclass: #FileSystemVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!FileSystemVisitor commentStamp: 'cwp 11/18/2009 12:25' prior: 0!
I am an abstract superclass for objects that can perform operations on directory trees. My subclasses implement the visitor protocol, and process filesystem nodes shown to them by guides.!

FileSystemVisitor subclass: #AbstractEnumerationVisitor
	instanceVariableNames: 'out block'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!AbstractEnumerationVisitor commentStamp: '<historical>' prior: 0!
I'm an abstract superclass for enumeration operations on directory entries. !

!AbstractEnumerationVisitor methodsFor: 'visiting' stamp: 'EstebanLorenzano 4/2/2012 11:37'!
breadthFirst: aReference
	^ self visit: aReference with: (BreadthFirstGuide for: self)! !

!AbstractEnumerationVisitor methodsFor: 'initialization' stamp: 'CamilloBruni 8/9/2011 15:46'!
initializeWithBlock: aBlock
	self initialize.
	block := aBlock! !

!AbstractEnumerationVisitor methodsFor: 'visiting' stamp: 'EstebanLorenzano 4/2/2012 11:38'!
postorder: aReference
	^ self visit: aReference with: (PostorderGuide for: self)! !

!AbstractEnumerationVisitor methodsFor: 'visiting' stamp: 'EstebanLorenzano 4/2/2012 11:38'!
preorder: aReference
	^ self visit: aReference with: (PreorderGuide for: self)! !

!AbstractEnumerationVisitor methodsFor: 'visiting' stamp: 'CamilloBruni 8/9/2011 15:46'!
visit: aReference with: aGuide
	out := (Array new: 10) writeStream.
	aGuide show: aReference.
	^ out contents! !

!AbstractEnumerationVisitor methodsFor: 'visiting' stamp: 'CamilloBruni 8/9/2011 15:48'!
visitReference: anEntry
	self subclassResponsibility! !

AbstractEnumerationVisitor subclass: #CollectVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!CollectVisitor commentStamp: 'cwp 11/18/2009 12:32' prior: 0!
I am a visitor that collects objects from the nodes I visit. I take a block similar to those passed to Collection>>collect:. I evaluate the block with DirectoryEntries for the nodes I visit, and collect the objects answered into an array.

I can use any guide, and the objects in the array I produce will reflect the order imposed by the guide.!

!CollectVisitor class methodsFor: 'instance creation' stamp: 'cwp 11/16/2009 10:33'!
breadthFirst: aReference
	^ self breadthFirst: aReference collect: [:entry | entry]! !

!CollectVisitor class methodsFor: 'instance creation' stamp: 'cwp 11/16/2009 10:32'!
breadthFirst: aReference collect: aBlock
	^ (self collect: aBlock) breadthFirst: aReference! !

!CollectVisitor class methodsFor: 'instance creation' stamp: 'cwp 11/16/2009 10:39'!
collect: aBlock
	^ self basicNew initializeWithBlock: aBlock! !

!CollectVisitor class methodsFor: 'instance creation' stamp: 'cwp 11/16/2009 10:36'!
postorder: aReference
	^ self postorder: aReference collect: [:entry | entry]! !

!CollectVisitor class methodsFor: 'instance creation' stamp: 'cwp 11/16/2009 10:35'!
postorder: aReference collect: aBlock
	^ (self collect: aBlock) postorder: aReference! !

!CollectVisitor class methodsFor: 'instance creation' stamp: 'cwp 11/16/2009 10:35'!
preorder: aReference
	^ self preorder: aReference collect: [:entry | entry]! !

!CollectVisitor class methodsFor: 'instance creation' stamp: 'cwp 11/16/2009 10:36'!
preorder: aReference collect: aBlock
	^ (self collect: aBlock) preorder: aReference! !

!CollectVisitor methodsFor: 'visiting' stamp: 'cwp 11/16/2009 10:38'!
visitReference: anEntry
	out nextPut: (block value: anEntry)! !

AbstractEnumerationVisitor subclass: #SelectVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!SelectVisitor commentStamp: '<historical>' prior: 0!
I am a visitor that selects objects from the nodes I visit. I take a block similar to those passed to Collection>>select:. I evaluate the block with DirectoryEntries for the nodes I visit.

I can use any guide, and the objects in the array I produce will reflect the order imposed by the guide.!

!SelectVisitor class methodsFor: 'instance creation' stamp: 'CamilloBruni 8/9/2011 15:50'!
breadthFirst: aReference
	^ self breadthFirst: aReference select: [:entry | true]! !

!SelectVisitor class methodsFor: 'instance creation' stamp: 'CamilloBruni 8/9/2011 15:50'!
breadthFirst: aReference select: aBlock
	^ (self select: aBlock) breadthFirst: aReference! !

!SelectVisitor class methodsFor: 'instance creation' stamp: 'CamilloBruni 8/9/2011 15:50'!
postorder: aReference
	^ self postorder: aReference select: [:entry | true]! !

!SelectVisitor class methodsFor: 'instance creation' stamp: 'CamilloBruni 8/9/2011 15:50'!
postorder: aReference select: aBlock
	^ (self select: aBlock) postorder: aReference! !

!SelectVisitor class methodsFor: 'instance creation' stamp: 'CamilloBruni 8/9/2011 15:50'!
preorder: aReference
	^ self preorder: aReference select: [:entry | true]! !

!SelectVisitor class methodsFor: 'instance creation' stamp: 'CamilloBruni 8/9/2011 15:50'!
preorder: aReference select: aBlock
	^ (self select: aBlock) preorder: aReference! !

!SelectVisitor class methodsFor: 'instance creation' stamp: 'CamilloBruni 8/9/2011 15:50'!
select: aBlock
	^ self basicNew initializeWithBlock: aBlock! !

!SelectVisitor methodsFor: 'visiting' stamp: 'CamilloBruni 8/9/2011 15:48'!
visitReference: anEntry
	(block value: anEntry) 
		ifTrue: [ out nextPut: anEntry ]! !

FileSystemVisitor subclass: #CopyVisitor
	instanceVariableNames: 'source dest'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!CopyVisitor commentStamp: 'cwp 11/18/2009 12:30' prior: 0!
I create a copy of the directory tree that I visit. I use the PreorderGuide so that I can create directories before creating their contents. !

!CopyVisitor class methodsFor: 'instance creation' stamp: 'cwp 10/30/2009 13:44'!
copy: source to: dest
	(self from: source to: dest) visit! !

!CopyVisitor class methodsFor: 'instance creation' stamp: 'cwp 10/30/2009 13:41'!
from: srcReference to: dstReference
	^ self basicNew
		initializeWithSource: srcReference 
		dest: dstReference! !

!CopyVisitor methodsFor: 'visiting' stamp: 'cwp 11/17/2009 21:06'!
copyDirectory: aReference
	| directory |
	directory := dest resolve: (aReference relativeTo: source).
	directory createDirectory! !

!CopyVisitor methodsFor: 'visiting' stamp: 'cwp 11/15/2009 00:31'!
copyFile: aReference
	| copy |
	copy := dest resolve: (aReference relativeTo: source).
	aReference copyTo: copy! !

!CopyVisitor methodsFor: 'initialize-release' stamp: 'cwp 10/30/2009 13:42'!
initializeWithSource: srcReference dest: dstReference
	self initialize.
	source := srcReference.
	dest := dstReference! !

!CopyVisitor methodsFor: 'visiting' stamp: 'EstebanLorenzano 4/2/2012 11:38'!
visit
	(PreorderGuide for: self) 
		show: source! !

!CopyVisitor methodsFor: 'visiting' stamp: 'cwp 11/16/2009 10:51'!
visitDirectory: anEntry
	| reference |
	reference := anEntry reference.
	reference = source
		ifTrue: [dest ensureDirectory]
		ifFalse: [self copyDirectory: reference]! !

!CopyVisitor methodsFor: 'visiting' stamp: 'cwp 11/16/2009 10:52'!
visitFile: anEntry
	| reference |
	reference := anEntry reference.
	reference = source
		ifTrue: [source copyTo: dest]
		ifFalse: [self copyFile: reference]! !

FileSystemVisitor subclass: #DeleteVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!DeleteVisitor commentStamp: 'cwp 11/18/2009 12:30' prior: 0!
I delete the directory tree that I visit. I use the PostorderGuide so that I can delete files before deleting their containing directories.!

!DeleteVisitor class methodsFor: 'instance creation' stamp: 'cwp 11/17/2009 13:02'!
delete: aReference
	^ self new visit: aReference! !

!DeleteVisitor methodsFor: 'visiting' stamp: 'EstebanLorenzano 4/2/2012 11:38'!
visit: aReference
	PostorderGuide show: aReference to: self! !

!DeleteVisitor methodsFor: 'visiting' stamp: 'cwp 11/16/2009 10:53'!
visitReference: anEntry
	anEntry reference delete! !

!FileSystemVisitor methodsFor: 'visiting' stamp: 'cwp 11/15/2009 00:45'!
visitDirectory: aReference
	^ self visitReference: aReference! !

!FileSystemVisitor methodsFor: 'visiting' stamp: 'cwp 11/15/2009 00:45'!
visitFile: aReference
	^ self visitReference: aReference! !

!FileSystemVisitor methodsFor: 'visiting' stamp: 'cwp 11/15/2009 00:45'!
visitReference: aReference! !

Object variableSubclass: #Path
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!Path commentStamp: '<historical>' prior: 0!
I'm a private and abstract filesystem path, independent of the string representation used to describe paths on a specific filesystem. I provide methods for navigating the filesystem hierarchy and working with absolute and relative paths. I only refer to a concrete file or directory with regard to a specific filesystem. Normally users should not use me directly. 

Path independent representation of delimiter is defined by DiskFileSystem current delimiter.

API instance creation:
#* and #/ are mnemonic to . and /
whose arguments should  be string file- or directory names, not fragments of Unix path notation intended to be parsed.

#/ and #* provide a mini-DSL for building up paths, while
#readFrom:delimiter: parses path strings.

Note that (Path with: 'parent/child/') isRelative returns true
because it creates to a relative path to a file/directory called
'parent/child'. In bash you'd escape the slashes like this: parent\/child\/

similarly 
(Path with: '/parent/child/') isRelative returns true
That's a relative path to '/parent/child'. In bash: /\parent\/child\/

(Path with: '') isRelative returns false

Because this is an absolute path to the root of the file system. Absolute paths
have an empty first element. If you consider $/ the separator,
'/usr/local/bin' has an empty first element.

!

Path variableSubclass: #AbsolutePath
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!AbsolutePath commentStamp: '<historical>' prior: 0!
I represent an absolute path (a position starting from Path root)!

!AbsolutePath class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 1/19/2012 15:12'!
addEmptyElementTo: result! !

!AbsolutePath class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 1/19/2012 15:02'!
from: aString delimiter: aDelimiterCharater
	aString = '/'
		ifTrue: [ ^ self root ].
		
	^ super from: aString delimiter: aDelimiterCharater! !

!AbsolutePath methodsFor: 'testing' stamp: 'cwp 2/26/2011 10:58'!
isAbsolute
	^ true! !

!AbsolutePath methodsFor: 'testing' stamp: 'cwp 2/26/2011 11:03'!
isRoot
	^ self size = 0! !

!AbsolutePath methodsFor: 'printing' stamp: 'EstebanLorenzano 4/3/2012 11:15'!
printOn: aStream
	aStream nextPutAll: 'Path'.
	self isRoot
		ifTrue: [aStream nextPutAll: ' root']
		ifFalse:
			[1 to: self size do:
				[:i |
				aStream 
					nextPutAll: ' / ''';
				 	nextPutAll: (self at: i);
					nextPut: $']]! !

!AbsolutePath methodsFor: 'enumerating' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
withParents
	^ super withParents addFirst: (Path root); yourself! !

!Path class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
* aString
	"Answer a relative path with aString as its sole segment. For example,
	FSPath * 'griffle' will produce the same result as ./griffle in a unix shell.
	The selector #* was chosen for it's visual similarity to $."
	
	"Note: aString is not parsed, so supplying a string like '/griffle/plonk'
	will not create an absolute path."
	
	^ RelativePath with: aString! !

!Path class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
/ aString
	"Answer an absolute path with aString as it's sole segment. The selector
	was chosen to allow path construction with Smalltalk syntax, which 
	neverthelesss resembles paths as they appear in a unix shell. Eg.
	FSPath / 'griffle' / 'plonk'."
	
	aString isEmptyOrNil 
		ifTrue: [ Error signal: 'Path element cannot be empty or nil'].
	^ AbsolutePath with: aString! !

!Path class methodsFor: 'private' stamp: 'CamilloBruni 1/19/2012 15:03'!
addElement: element to: result
	element = '..' ifTrue: [^ self addParentElementTo: result].
	element = ''   ifTrue: [^ self addEmptyElementTo: result].
	element = '.'  ifFalse: [result add: element]! !

!Path class methodsFor: 'private' stamp: 'cwp 10/26/2009 13:41'!
addEmptyElementTo: result
	result isEmpty ifTrue: [result add: '']
		! !

!Path class methodsFor: 'private' stamp: 'cwp 10/26/2009 13:39'!
addParentElementTo: result
	result isEmpty
		ifTrue: [result add: '..']
		ifFalse: [result removeLast]
		! !

!Path class methodsFor: 'private' stamp: 'cwp 10/26/2009 13:30'!
canonicalizeElements: aCollection
	| result |
	result := OrderedCollection new.
	aCollection do: [:element | self addElement: element to: result].
	^ result! !

!Path class methodsFor: 'encodings' stamp: 'StephaneDucasse 2/18/2011 22:31'!
extensionDelimiter
	"Return the extension delimiter character."
	^ $.! !

!Path class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/7/2012 02:19'!
from: aString
	^ self from: aString delimiter: $/! !

!Path class methodsFor: 'instance creation' stamp: 'tbn 6/26/2012 21:03'!
from: aString delimiter: aDelimiterCharacter 
	"Answer a path composed of several elements delimited by aCharacter"
	| pathCls |
	aString isEmpty
		ifTrue: [ ^ self root ].
	
	pathCls :=  ((self isAbsoluteUnixPath: aString) or: [self isAbsoluteWindowsPath: aString]) 
		ifTrue: [AbsolutePath]
		ifFalse:[RelativePath].
	
	^ pathCls withAll: (pathCls canonicalizeElements: (aDelimiterCharacter split: aString))! !

!Path class methodsFor: 'private' stamp: 'tbn 6/26/2012 21:02'!
isAbsoluteUnixPath: aString
	^aString first = $/ ! !

!Path class methodsFor: 'private' stamp: 'tbn 6/26/2012 21:02'!
isAbsoluteWindowsPath: aString
	^aString matchesRegex: '[a-zA-Z]\:\\.*'! !

!Path class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
parent
	"Answer a path that resolves to the parent of the current
	working directory. This is similar to .. in unix, but doesn't
	rely on actual hardlinks being present in the filesystem."

	^ RelativePath with: '..'! !

!Path class methodsFor: 'instance creation' stamp: 'cwp 11/15/2009 00:11'!
parents: anInteger
	| path |
	path := self new: anInteger.
	1 to: anInteger do: [:i | path at: i put: '..'].
	^ path! !

!Path class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
root
	"Answer the root path - ie, / on unix"
	
	^ AbsolutePath new! !

!Path class methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:42'!
with: aString
	"Answer a relative path of the given string. N.B. that the argument is not parsed; 
	it is the name of a single path element, and path separators in it do not have 
	special meaning."
	
	"(FSPath with: '/parent/child/') isRelative
	answers true because this is a relative path to a file or directory named 
	'/parent/child/'. In bash: \/parent\/child\/"

	| inst |
	inst := self new: 1.
	inst at: 1 put: aString.
	^ inst! !

!Path class methodsFor: 'private' stamp: 'CamilloBruni 5/8/2012 17:47'!
withAll: aCollection
	| inst |
	inst := self new: aCollection size.
	aCollection withIndexDo: [:segment :index |
		inst at: index put: segment].
	^ inst! !

!Path class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
workingDirectory
	"Answer a path that will always resolve to the current working directory."
	
	^ RelativePath new! !

!Path methodsFor: 'navigating' stamp: 'cwp 11/17/2009 23:52'!
, extension 
	^ self withName: self basename extension: extension! !

!Path methodsFor: 'navigating' stamp: 'CamilloBruni 8/15/2011 17:36'!
/ aString
	| path |

	aString isEmptyOrNil 
		ifTrue: [ Error signal: 'Path element cannot be empty or nil'].
		
	path := self class new: self size + 1.
	path copyFrom: self.
	path at: path size put: aString.
	^ path! !

!Path methodsFor: 'comparing' stamp: 'cwp 12/14/2008 17:36'!
= other
	^ self species = other species
		and: [self size = other size
			and: [(1 to: self size) allSatisfy: [:i | (self at: i) = (other at: i)]]]! !

!Path methodsFor: 'converting' stamp: 'EstebanLorenzano 4/12/2012 14:28'!
asFileReference
	^ FileSystem disk referenceTo: self! !

!Path methodsFor: 'converting' stamp: 'cwp 10/10/2009 18:04'!
asPathWith: anObject
	^ self! !

!Path methodsFor: 'converting' stamp: 'EstebanLorenzano 4/12/2012 14:30'!
asReference
	self deprecated: 'Use #asFileReference instead'  on: '12 April 2012' in:  'Pharo 1.4' .
	^ self asFileReference! !

!Path methodsFor: 'navigating' stamp: 'cwp 9/22/2009 09:08'!
asResolvedBy: anObject
	^ anObject resolvePath: self! !

!Path methodsFor: 'converting' stamp: 'abc 5/11/2012 17:33'!
asUrl 
	^ FileUrl
		pathParts: self segments asOrderedCollection 
		isAbsolute: self isAbsolute! !

!Path methodsFor: 'accessing' stamp: 'StephaneDucasse 2/15/2010 18:06'!
base
	"Returns the base of the basename, i.e. /foo/gloops.taz basename is 'gloops'"
	^ self basename copyUpTo: self extensionDelimiter! !

!Path methodsFor: 'accessing' stamp: 'CamilloBruni 8/9/2011 16:11'!
basename
	"Returns the base of the basename, 
		i.e. 
		/foo/gloops.taz basename is 'gloops.taz'
		/ basename is '/'"
	self size == 0
		"the root node"
		ifTrue: [ ^ '/'].
	^ self at: self size! !

!Path methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 17:41'!
basename: newBasename
	"change the basename"
	self size == 0
		"the root node"
		ifTrue: [ ^ Error signal: '0 length Path, cannot change basename'].
	self at: self size put: newBasename! !

!Path methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2012 18:22'!
basenameWithoutExtension
	"Returns the base of the basename but without its extension, 
		i.e. 
		/foo/gloops.taz basenameWithoutExtension is 'gloops'
		/ basenameWithoutExtension is '/'"
	
	^ self basename copyUpToLast: self extensionDelimiter! !

!Path methodsFor: 'comparing' stamp: 'cwp 10/26/2009 01:03'!
contains: anObject
	^ anObject isContainedBy: self! !

!Path methodsFor: 'comparing' stamp: 'cwp 10/25/2009 22:59'!
containsPath: aPath
	self size < aPath size ifFalse: [^ false].
	1	to: self size 
		do: [:i | (self at: i) = (aPath at: i) ifFalse: [^ false]].
	^ true! !

!Path methodsFor: 'comparing' stamp: 'cwp 10/25/2009 23:05'!
containsReference: aReference
	^ false! !

!Path methodsFor: 'private' stamp: 'cwp 12/13/2008 21:08'!
copyFrom: aPath
	| size |
	size := aPath size min: self size.
	1 to: size do: [:i | self at: i put: (aPath at: i)].
	! !

!Path methodsFor: 'accessing' stamp: 'cwp 10/11/2009 11:05'!
delimiter
	^ $/! !

!Path methodsFor: 'enumerating' stamp: 'cwp 7/18/2009 01:13'!
do: aBlock 
	1 
		to: self size
		do: 
			[ :index || segment |
			segment := self at: index.
			segment isEmpty ifFalse: [ aBlock value: segment ] ]! !

!Path methodsFor: 'accessing' stamp: 'SeanDeNigris 6/28/2012 00:09'!
extension
	"Return the extension of path basename i.e., /foo/gloops.taz extension is 'taz'"
	
	^ self basename copyAfterLast: self extensionDelimiter! !

!Path methodsFor: 'accessing' stamp: 'cwp 12/23/2008 11:25'!
extensionDelimiter
	^ self class extensionDelimiter! !

!Path methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2012 18:18'!
extensions
	"Return the extensions of the receiver in order of appearance"
	"(Path from: '/foo/bar.tar.gz') extensions"
	^ (self extensionDelimiter split: self basename) allButFirst! !

!Path methodsFor: 'accessing' stamp: 'sd 2/11/2011 21:02'!
fullName
	"Return the fullName of the receiver."
	
	^ self printString! !

!Path methodsFor: 'comparing' stamp: 'cwp 12/14/2008 17:06'!
hash
	| hash |
	hash := self class identityHash.
	1 to: self size do:
		[:i | hash := String stringHash: (self at: i) initialHash: hash].
	^ hash! !

!Path methodsFor: 'testing' stamp: 'cwp 2/26/2011 10:58'!
isAbsolute
	self subclassResponsibility ! !

!Path methodsFor: 'private' stamp: 'cwp 10/25/2009 19:53'!
isAllParents
	1 to: self size do: [:i | (self at: i) = '..' ifFalse: [^ false]].
	^ true! !

!Path methodsFor: 'comparing' stamp: 'cwp 11/16/2009 09:06'!
isChildOf: anObject
	^ self parent = anObject! !

!Path methodsFor: 'comparing' stamp: 'cwp 10/25/2009 23:01'!
isContainedBy: anObject
	^ anObject containsPath: self! !

!Path methodsFor: 'testing' stamp: 'DamienPollet 2/20/2011 04:00'!
isEmpty
	^ self size = 0! !

!Path methodsFor: 'testing' stamp: 'cwp 12/13/2008 21:00'!
isRelative
	^ self isAbsolute not! !

!Path methodsFor: 'testing' stamp: 'cwp 2/26/2011 11:03'!
isRoot
	self subclassResponsibility ! !

!Path methodsFor: 'testing' stamp: 'cwp 7/18/2009 00:42'!
isWorkingDirectory
	^ self size = 0! !

!Path methodsFor: 'private' stamp: 'CamilloBruni 8/12/2011 16:42'!
lengthOfStemWith: aPath
	| limit index |
	limit := self size min: aPath size.
	index := 1.
	[index <= limit and: [(self at: index) = (aPath at: index)]] 
		whileTrue: [index := index + 1].
	^ index - 1! !

!Path methodsFor: 'navigating' stamp: 'cwp 11/15/2009 00:00'!
makeRelative: anObject
	^ anObject relativeToPath: self! !

!Path methodsFor: 'navigating' stamp: 'cwp 10/25/2009 19:53'!
parent
	| size parent |
	self isRoot ifTrue: [^ self].
	self isAllParents ifTrue: [^ self / '..'].
	
	size := self size - 1.
	parent := self class new: size.
	1 to: size do: [:i | parent at: i put: (self at: i)].
	^ parent! !

!Path methodsFor: 'printing' stamp: 'cwp 11/17/2009 10:22'!
printOn: aStream 
	self printOn: aStream delimiter: self delimiter.
! !

!Path methodsFor: 'printing' stamp: 'cwp 2/26/2011 17:58'!
printOn: aStream delimiter: aCharacter
	(1 to: self size)
		do: [:index | aStream nextPutAll: (self at: index)]
		separatedBy: [aStream nextPut: aCharacter]! !

!Path methodsFor: 'printing' stamp: 'cwp 1/13/2009 21:27'!
printWithDelimiter: aCharacter
	^ String streamContents: [:out | self printOn: out delimiter: aCharacter]! !

!Path methodsFor: 'navigating' stamp: 'cwp 11/15/2009 00:00'!
relativeTo: anObject
	^ anObject makeRelative: self! !

!Path methodsFor: 'navigating' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
relativeToPath: aPath
	"Return the receiver as relative to the argument aPath"
	
	"(FSPath / 'griffle' / 'plonk' / 'nurp') 
		relativeToPath: (FSPath / 'griffle') 
			returns  plonk/nurp"

	| prefix relative |
	aPath isRelative ifTrue: [^ aPath].
	prefix := self lengthOfStemWith: aPath.
	relative := RelativePath parents: (aPath size - prefix).
	prefix + 1 to: self size do: [:i | relative := relative / (self at: i)].
	^ relative! !

!Path methodsFor: 'navigating' stamp: 'cwp 3/29/2011 16:23'!
relativeToReference: aReference 
	^ self relativeToPath: aReference path! !

!Path methodsFor: 'navigating' stamp: 'cwp 11/16/2009 10:19'!
resolve
	^ self! !

!Path methodsFor: 'navigating' stamp: 'cwp 3/29/2011 16:25'!
resolve: anObject
	"Return a path in which the argument has been interpreted in the context of the receiver. Different 
	argument types have different resolution semantics, so we use double dispatch to resolve them correctly."
	
	^ anObject asResolvedBy: self! !

!Path methodsFor: 'navigating' stamp: 'CamilloBruni 1/19/2012 14:31'!
resolvePath: aPath
	"Answers an absolute path created by resolving the argument against the receiver.
	If the argument is abolute answer the argument itself. Otherwise, concatenate the
	two paths, then process all parent references '..', and create a path with the remaining
	elements."
	
	| elements |
	
	aPath isAbsolute ifTrue: [^ aPath].
	elements := Array new: self size + aPath size.
	
	1 to: self size do: [:i | elements at: i put: (self at: i)].
	1 to: aPath size do: [:i | elements at: self size + i put: (aPath at: i)].
	
	^ self class withAll: (self class canonicalizeElements: elements)! !

!Path methodsFor: 'navigating' stamp: 'cwp 9/22/2009 09:06'!
resolveReference: aReference
	^ aReference! !

!Path methodsFor: 'navigating' stamp: 'cwp 3/29/2011 16:30'!
resolveString: aString
	"Treat strings as relative paths with a single element."
	
	^ self / aString! !

!Path methodsFor: 'accessing' stamp: 'CamilloBruni 8/12/2011 20:16'!
segments
	"return an array with all the path segements separated"
	| segments index |
	segments := Array new: self size.
	index := 1.
	self do: [ :part|
		segments at: index put: part.
		index := index + 1 ].
	
	^ segments! !

!Path methodsFor: 'navigating' stamp: 'cwp 11/17/2009 23:51'!
withExtension: extension 
	| basename name |
	basename := self basename.
	^ (basename endsWith: extension) 
		ifTrue: [ self ]
		ifFalse: 
			[name := basename copyUpToLast: self extensionDelimiter.
			self withName: name extension: extension]! !

!Path methodsFor: 'private' stamp: 'cwp 11/17/2009 23:58'!
withName: name extension: extension
	| basename |
	basename :=String streamContents:
		[:out |
		out nextPutAll: name.
		out nextPut: self extensionDelimiter.
		out nextPutAll: extension].
	^ self copy 
		at: self size put: basename;
		yourself! !

!Path methodsFor: 'enumerating' stamp: 'cwp 3/29/2011 16:42'!
withParents
	| paths |
	paths := OrderedCollection new.
	1 to: self size -1 do: [ :index | paths add: ((self class new: index) copyFrom: self) ].
	paths add: self.
	
	^ paths! !

Path variableSubclass: #RelativePath
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!RelativePath commentStamp: '<historical>' prior: 0!
I represent a relative path (a position starting from Path workingDirectory)!

!RelativePath methodsFor: 'testing' stamp: 'cwp 2/26/2011 10:58'!
isAbsolute
	^ false! !

!RelativePath methodsFor: 'testing' stamp: 'cwp 2/26/2011 11:03'!
isRoot
	^ false! !

!RelativePath methodsFor: 'printing' stamp: 'EstebanLorenzano 4/3/2012 11:15'!
printOn: aStream
	aStream nextPutAll: 'Path '.
	self isWorkingDirectory
		ifTrue: [aStream nextPutAll: 'workingDirectory']
		ifFalse: 
			[aStream 
				nextPutAll: '* ''';
				nextPutAll: (self at: 1) asString;
				nextPut: $'.
			2 to: self size do:
				[:i |
				aStream
					nextPutAll: ' / ''';
					nextPutAll: (self at: i);
					nextPut: $']]
					
			! !

!RelativePath methodsFor: 'printing' stamp: 'cwp 2/26/2011 18:00'!
printOn: aStream delimiter: aCharacter
	self isWorkingDirectory ifTrue: [aStream nextPut: $.. ^ self].
	super printOn: aStream delimiter: aCharacter! !
FileLocator initialize!
